package com.yonge.cooleshow.admin.controller;

import com.alibaba.fastjson.JSON;
import com.yonge.cooleshow.biz.dal.dto.search.HomeworkAdminSearch;
import com.yonge.cooleshow.biz.dal.entity.StudentCourseHomework;
import com.yonge.cooleshow.biz.dal.entity.SysUser;
import com.yonge.cooleshow.biz.dal.enums.CourseScheduleEnum;
import com.yonge.cooleshow.biz.dal.service.CourseHomeworkService;
import com.yonge.cooleshow.biz.dal.service.CourseScheduleService;
import com.yonge.cooleshow.biz.dal.service.StudentCourseHomeworkService;
import com.yonge.cooleshow.biz.dal.service.SysUserService;
import com.yonge.cooleshow.biz.dal.vo.CourseHomeworkDetailVo;
import com.yonge.cooleshow.biz.dal.vo.CourseHomeworkVo;
import com.yonge.cooleshow.biz.dal.wrapper.HomeworkWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@RequestMapping("${app-config.url.admin:}/homework")
@Api(value = "课程作业表", tags = "课程作业表")
public class CourseHomeworkController extends BaseController {

    @Resource
    private CourseHomeworkService courseHomeworkService;

	@Resource
	private CourseScheduleService courseScheduleService;

	@Resource
	private StudentCourseHomeworkService studentCourseHomeworkService;
	@Resource
	private SysUserService sysUserService;

	@ApiOperation(value = "课后作业-列表", httpMethod = "POST", consumes = "application/json", produces = "application/json")
	@PostMapping(value = "/list", consumes = "application/json", produces = "application/json")
	@PreAuthorize("@pcs.hasPermissions('homework/list')")
	public HttpResponseResult<PageInfo<CourseHomeworkVo>> list(@Valid @RequestBody HomeworkAdminSearch query) {
		query.setDecorate(YesOrNoEnum.YES);
		query.setCourseStatus(CourseScheduleEnum.COMPLETE);

		List<CourseScheduleEnum> list = new ArrayList<>();
		list.add(CourseScheduleEnum.PIANO_ROOM_CLASS);
		list.add(CourseScheduleEnum.PRACTICE);
		query.setCourseType(list);
		return succeed(PageUtil.pageInfo(courseHomeworkService.selectAdminPage(PageUtil.getPage(query),query)));
	}


	@ApiOperation(value = "课后作业-详情",notes = "传入课程编号ID")
	@GetMapping(value = "/detail/{courseId}/{studentId}")
	@PreAuthorize("@pcs.hasPermissions('homework/detail')")
	public HttpResponseResult<CourseHomeworkDetailVo> detail(@ApiParam(value = "课程编号ID", required = true)
															 @PathVariable("courseId") Long courseId,
															 @PathVariable("studentId") Long studentId) {
		return succeed(courseHomeworkService.getCourseHomeworkDetailByCourseId(courseId,studentId));
	}



	@PostMapping(value = "/teacherSend")
	@ApiOperation(value = "发送老师未评价和未布置作业消息")
	@PreAuthorize("@pcs.hasPermissions('homework/teacherSend')")
	public HttpResponseResult<Object> sendTodayNotRepliedAndNotDecorateHomework() {
		courseScheduleService.sendTodayNotRepliedAndNotDecorateHomework();
		return HttpResponseResult.succeed();
	}

	//课程组关联的作业列表
	@ApiOperation(value = "课程组关联的作业列表", httpMethod = "POST", consumes = "application/json", produces = "application/json")
	@PostMapping(value = "/groupList", consumes = "application/json", produces = "application/json")
	@PreAuthorize("@pcs.hasPermissions('homework/groupList')")
	public HttpResponseResult<List<HomeworkWrapper.GroupHomework>> groupList(@RequestBody HomeworkWrapper.GroupHomeworkSearch query) {
		return succeed(courseHomeworkService.groupList(query));
	}

	//根据作业编号，获取学员作业列表
	@ApiOperation(value = "根据作业编号，获取学员作业列表")
	@GetMapping(value = "/studentList")
	@PreAuthorize("@pcs.hasPermissions('homework/studentList')")
	public HttpResponseResult<List<HomeworkWrapper.StudentCourseHomeworkDto>> studentList(Long homeworkId) {
		List<StudentCourseHomework> list = studentCourseHomeworkService.lambdaQuery().eq(StudentCourseHomework::getCourseHomeworkId, homeworkId).list();
		if (CollectionUtils.isNotEmpty(list)){
			List<HomeworkWrapper.StudentCourseHomeworkDto> dtos = JSON.parseArray(JSON.toJSONString(list), HomeworkWrapper.StudentCourseHomeworkDto.class);
			List<Long> studentIds = dtos.stream().map(StudentCourseHomework::getStudentId).collect(Collectors.toList());
			List<SysUser> sysUsers = sysUserService.getDao().selectBatchIds(studentIds);
			Map<Long, String> nameMap = sysUsers.stream().collect(Collectors.toMap(SysUser::getId, SysUser::getUsername));
			dtos.forEach(e -> e.setStudentName(nameMap.get(e.getStudentId())));
			return succeed(dtos);
		}
		return succeed(new ArrayList<>());
	}
}