package com.yonge.cooleshow.admin.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.microsvc.toolkit.common.response.template.R;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.service.DiscountCardRecordService;
import com.yonge.cooleshow.biz.dal.service.SysUserService;
import com.yonge.cooleshow.biz.dal.wrapper.DiscountCardRecordWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.toolset.base.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("${app-config.url.admin:}/discountCardRecord")
@Api(value = "畅学卡记录", tags = "畅学卡记录")
public class DiscountCardRecordController extends BaseController {

    @Autowired
    private DiscountCardRecordService discountCardRecordService;

    @Autowired
    private SysUserService sysUserService;

    @PostMapping("/page")
    @ApiOperation(value = "畅学卡记录列表")
    public R<PageInfo<DiscountCardRecordWrapper.DiscountCardRecord>> page(@RequestBody DiscountCardRecordWrapper.DiscountCardRecordQuery query) {

        // 查询数据
        IPage<DiscountCardRecordWrapper.DiscountCardRecord> pages = discountCardRecordService.selectPage(QueryInfo.getPage(query), query);


        return R.from(QueryInfo.pageInfo(pages));
    }

    @ApiOperation(value = "新增畅学卡", notes = "传入vipPurchaseRecord")
    @PostMapping("/add")
    public R<JSONObject> add(@Validated @RequestBody DiscountCardRecordWrapper.AddDiscountCardRecord result) {
        SysUser sysUser = sysUserService.getUser();
        if (sysUser == null  || sysUser.getId() == null) {
            throw new BizException( "用户信息获取失败");
        }
        result.setCreateBy(sysUser.getId());
        discountCardRecordService.addStudentVip(result);
        return R.defaultR();
    }



    @ApiOperation(value = "扣减畅学卡", notes = "传入vipPurchaseRecord")
    @PostMapping("/deduction")
    public R<JSONObject> deduction(@Validated @RequestBody DiscountCardRecordWrapper.DeductionDiscountCardRecord result) {
        SysUser sysUser = sysUserService.getUser();
        if (sysUser == null  || sysUser.getId() == null) {
            throw new BizException( "用户信息获取失败");
        }
        result.setCreateBy(sysUser.getId());
        discountCardRecordService.deduction(result);
        return R.defaultR();
    }





    @ApiOperation(value = "畅学卡时长")
    @PostMapping("/info")
    public R<DiscountCardRecordWrapper.Info> info(@RequestBody DiscountCardRecordWrapper.InfoQuery infoQuery) {

        return R.from(discountCardRecordService.info(infoQuery));
    }
}