package com.yonge.cooleshow.admin.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.microsvc.toolkit.common.response.template.R;
import com.yonge.cooleshow.admin.io.request.UserTenantAlbumRecordVo;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.entity.UserTenantAlbumRecord;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.SourceTypeEnum;
import com.yonge.cooleshow.biz.dal.service.SysUserService;
import com.yonge.cooleshow.biz.dal.service.UserTenantAlbumRecordService;
import com.yonge.cooleshow.biz.dal.wrapper.UserTenantAlbumRecordWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.toolset.base.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.admin:}/userTenantAlbumRecord")
@Api(tags = "购买训练工具记录")
public class UserTenantAlbumRecordController extends BaseController {

    @Autowired
    private UserTenantAlbumRecordService userTenantAlbumRecordService;

    @Resource
    private SysUserService sysUserService;

    @Autowired
    private RedissonClient redissonClient;

    @PostMapping("/page")
    @ApiOperation(value = "记录列表")
    public R<PageInfo<UserTenantAlbumRecordWrapper.AdminUserTenantAlbumRecord>> page(@RequestBody UserTenantAlbumRecordWrapper.UserTenantAlbumRecordQuery query) {

        // 查询数据
        IPage<UserTenantAlbumRecordWrapper.AdminUserTenantAlbumRecord> pages = userTenantAlbumRecordService.selectAdminPage(QueryInfo.getPage(query), query);


        return R.from(QueryInfo.pageInfo(pages));
    }


    @ApiOperation(value = "新增", notes = "购买训练工具记录- 传入 UserTenantAlbumRecordVo.UserTenantAlbumRecord")
    @PreAuthorize("@pcs.hasPermissions('userTenantAlbumRecord/save', {'BACKEND'})")
    @PostMapping("/save")
    public R<JSONObject> add(@Validated @RequestBody UserTenantAlbumRecordVo.UserTenantAlbumRecord userTenantAlbumRecordVo) {

        // 新增数据
        UserTenantAlbumRecord userTenantAlbumRecord = new UserTenantAlbumRecord();
        userTenantAlbumRecord.setTenantId(userTenantAlbumRecordVo.getTenantId());
        userTenantAlbumRecord.setTenantAlbumId(userTenantAlbumRecordVo.getTenantAlbumId());
        userTenantAlbumRecord.setTimes(userTenantAlbumRecordVo.getTimes());
        userTenantAlbumRecord.setType(userTenantAlbumRecordVo.getType());
        userTenantAlbumRecord.setReason(userTenantAlbumRecordVo.getReason());
        userTenantAlbumRecord.setUserId(userTenantAlbumRecordVo.getUserId());
        userTenantAlbumRecord.setClientType(ClientEnum.STUDENT);
        userTenantAlbumRecord.setSourceType(SourceTypeEnum.BACKEND_GIVE);

        SysUser sysUser = sysUserService.getUser();
        userTenantAlbumRecord.setCreateBy(sysUser.getId());

        RLock lock = redissonClient.getLock("UserTenantAlbumRecord_" + userTenantAlbumRecordVo.getUserId() + "_" + userTenantAlbumRecordVo.getTenantAlbumId());
        try {
            boolean b = lock.tryLock(60, 60, TimeUnit.SECONDS);
            if (b) {
                userTenantAlbumRecordService.add(userTenantAlbumRecord,userTenantAlbumRecordVo.getSendMsg());
            } else {
                throw new BizException("请勿重复点击");
            }
        } catch (InterruptedException e) {
            log.error("获取锁异常", e);
        } finally {
            if (lock.getHoldCount() > 0) {
                lock.unlock();
            }
        }
        return R.defaultR();
    }


    @ApiOperation(value = "扣减", notes = "传入vipPurchaseRecord")
    @PostMapping("/deduction")
    public R<JSONObject> deduction(@Validated @RequestBody UserTenantAlbumRecordWrapper.DeductionUserTenantAlbumRecord result) {
        SysUser sysUser = sysUserService.getUser();
        if (sysUser == null  || sysUser.getId() == null) {
            throw new BizException( "用户信息获取失败");
        }
        result.setCreateBy(sysUser.getId());
        userTenantAlbumRecordService.deduction(result);
        return R.defaultR();
    }





    @ApiOperation(value = "时长")
    @PostMapping("/info")
    public R<List<UserTenantAlbumRecordWrapper.Info>> info(@RequestBody UserTenantAlbumRecordWrapper.InfoQuery infoQuery) {

        return R.from(userTenantAlbumRecordService.info(infoQuery));
    }
}