package com.yonge.cooleshow.student.task;

import com.yonge.cooleshow.biz.dal.entity.StudentTotal;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.payment.util.DistributedLock;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @Author: liweifan
 * @Data: 2022/3/28 15:50
 */
@RestController
@RequestMapping("${app-config.url.student:}/task")
@ApiIgnore
@Slf4j
public class TaskController extends BaseController {
    @Resource
    private StudentTotalService studentTotalService;
    @Resource
    private StudentTimeService studentTimeService;
    @Resource
    private VipCardRecordService vipCardRecordService;
    @Resource
    private DiscountCardRecordService discountCardRecordService;
    @Resource
    private CourseRepliedService repliedService;


    @Autowired
    private RedissonClient redissonClient;
    /***
     * 查询所有学生统计数据
     * @author liweifan
     * @updateTime 2022/4/29 19:12
     * @return: com.yonge.cooleshow.common.entity.HttpResponseResult<java.util.List<com.yonge.cooleshow.biz.dal.entity.StudentTotal>>
     */
    @GetMapping("/queryStudentTotal")
    public HttpResponseResult<List<StudentTotal>> queryStudentTotal() {
        return succeed(studentTotalService.queryStudentTotal());
    }

    /***
     * 半小时一次
     * @author liweifan
     * @updateTime 2022/4/29 19:12
     * @return: com.yonge.cooleshow.common.entity.HttpResponseResult<java.lang.Boolean>
     */
    @GetMapping("/halfHourTask")
    public HttpResponseResult<Boolean> halfHourTask() {
        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("halfHourTask", () -> {
            try {
                //会员卡到期提醒
                vipCardRecordService.pollExpireMsg();
                //畅学卡到期提醒
                discountCardRecordService.pollExpireMsg();
            }catch (Exception e){
                log.error("halfHourTask error", e);
            }
        }, 10L, TimeUnit.SECONDS);
        return succeed(true);
    }

    /***
     * 统计学员第一次购买时间
     *
     * @author liweifan
     * @updateTime 2022/4/29 19:12
     * @return: com.yonge.cooleshow.common.entity.HttpResponseResult<java.lang.Boolean>
     */
    @GetMapping("/studentFirstBuy")
    public HttpResponseResult<Boolean> studentFirstBuy() {
        //统计学员第一次购买时间
        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("studentFirstBuy", () -> {
            try {

                studentTimeService.totalStudentTime();
            }catch (Exception e){
                log.error("studentFirstBuy error", e);
            }
        }, 10L, TimeUnit.SECONDS);
        return succeed(true);
    }

    /**
     * @Description: 当日趣纠课未对老师评价（每晚9点，已评价不发）
     * @Author: cy
     * @Date: 2022/5/6
     */
    @GetMapping("/noRepliedTeacher")
    public HttpResponseResult noRepliedTeacher() {

        DistributedLock.of(redissonClient).runIfLockCanGet("noRepliedTeacher", () -> {
            try {

                repliedService.noRepliedTeacher();
            }catch (Exception e){
                log.error("noRepliedTeacher error", e);
            }
        }, 10L, TimeUnit.SECONDS);
        return succeed();
    }
}
