package com.yonge.cooleshow.teacher.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.microsvc.toolkit.common.webportal.exception.BizException;
import com.microsvc.toolkit.middleware.oss.OssPluginContext;
import com.microsvc.toolkit.middleware.oss.impl.TencentOssPlugin;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.PianoRoomTimeVo;
import com.yonge.cooleshow.biz.dal.vo.TeacherHomeStatisticalVo;
import com.yonge.cooleshow.biz.dal.wrapper.TeacherIndexWrapper;
import com.yonge.cooleshow.biz.dal.wrapper.teacher.TeacherHomeWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import com.yonge.toolset.utils.date.DateUtil;
import com.yonge.toolset.utils.easyexcel.ExcelUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.File;
import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Description 老师课后作业相关接口
 *
 * @author liujunchi
 * @date 2022-04-13
 */
@Api(tags = "老师APP首页接口")
@RestController
@RequestMapping("${app-config.url.teacher:}/home")
public class TeacherHomeController extends BaseController {

    @Resource
    private SysUserService sysUserService;
    @Resource
    private CourseScheduleService courseScheduleService;
    @Resource
    private MusicSheetService musicSheetService;
    @Resource
    private UserAccountRecordService userAccountRecordService;
    @Resource
    private HomeService homeService;
    @Resource
    private OssPluginContext ossPluginContext;

    @ApiOperation(value = "首页统计数据")
    @GetMapping(value="/count")
    public HttpResponseResult<TeacherHomeStatisticalVo> countTeacherHome() {
        Long userId = sysUserService.getUserId();

        // 我的课程
        Integer courseSechedule  = courseScheduleService.getWeekNotStart(userId);

        // 课后作业
        Integer courseHomework = courseScheduleService.getHomeworkNotDecorate(userId);

        // 课后评价
        Integer courseScheduleReplied = courseScheduleService.getNotRepliedCourseSchedule(userId);

        // 我的乐谱
        Integer musicSheet = musicSheetService.getTeacherMusicSheetCount(userId);

        // 我收到的评价
        Integer studentReplied = courseScheduleService.getWeekStudentRepliedCourseSchedule(userId);

        // 我的收入
        BigDecimal decimal = userAccountRecordService.getMonthDecimal(userId);


        // 琴房剩余时长
        PianoRoomTimeVo pianoRoomTimeVo = courseScheduleService.selectRemainTime(userId);

        TeacherHomeStatisticalVo teacherHomeStatisticalVo = new TeacherHomeStatisticalVo();
        teacherHomeStatisticalVo.setCourseHomework(courseHomework);
        teacherHomeStatisticalVo.setCourseScheduleReplied(courseScheduleReplied);
        teacherHomeStatisticalVo.setCourseSchedule(courseSechedule);
        teacherHomeStatisticalVo.setDecimal(decimal);
        teacherHomeStatisticalVo.setMusicSheet(musicSheet);
        teacherHomeStatisticalVo.setStudentReplied(studentReplied);
        teacherHomeStatisticalVo.setPianoTime(pianoRoomTimeVo.getRemainTime());
        return succeed(teacherHomeStatisticalVo);
    }

    @ApiOperation("首页练习数据统计")
    @PostMapping(value = "/practice")
    public HttpResponseResult<TeacherIndexWrapper.TeacherPracticeHome> practice(@RequestBody TeacherIndexWrapper.SummarySearch summarySearch) {
        summarySearch.setTeacherId(sysUserService.getUserId());
        return succeed(homeService.getTeacherPracticeHome(summarySearch));
    }

    @ApiOperation("首页练习数据学员列表")
    @PostMapping(value = "/studentPractice")
    public HttpResponseResult<List<TeacherIndexWrapper.StudentPracticeSummaryDto>> studentPractice(@RequestBody TeacherIndexWrapper.StudentSearch studentSearch) {
        studentSearch.setTeacherId(sysUserService.getUserId());
        return succeed(homeService.getTeacherHomeStudent(studentSearch));
    }

    @ApiOperation(value = "课程统计")
    @PostMapping("/totalCourse")
    public HttpResponseResult<List<TeacherHomeWrapper.TeacherTotalCourse>> totalCourse(@RequestBody TeacherHomeWrapper.TeacherTotalCourseQuery query) {
        query.setTeacherId(sysUserService.getUserId());

        return succeed(homeService.teacherTotalCourse(query));
    }


    @ApiOperation(value = "我的乐谱统计")
    @GetMapping("/musicSheetTotal")
    public HttpResponseResult<TeacherHomeWrapper.MusicSheetTotal> musicSheetTotal() {
        return succeed(homeService.musicSheetTotal(sysUserService.getUserId()));
    }

    @ApiOperation(value = "我的乐谱分页")
    @PostMapping("/musicSheetPage")
    public HttpResponseResult<List<TeacherHomeWrapper.MusicSheetTotal>> musicSheetPage(@RequestBody TeacherHomeWrapper.MusicSheetQuery query) {
        query.setTeacherId(sysUserService.getUserId());
        return succeed(homeService.musicSheetPage(query));
    }



    @ApiOperation(value = "课程浏览/购买")
    @PostMapping("/courseExposure")
    public HttpResponseResult<Map<String, List<TeacherIndexWrapper.CourseExposureTotal>>> courseExposure(@RequestBody TeacherIndexWrapper.CourseExposureSearch query) {
        query.setTeacherId(sysUserService.getUserId());
        Map<String, List<TeacherIndexWrapper.CourseExposureTotal>> result = new HashMap<>(2);
        //获取购买的统计数据
        result.put("buy", homeService.courseBuyTotal(query));
        //获取曝光的统计数据
        result.put("exposure", homeService.courseExposureTotal(query));
        return succeed(result);
    }

    @ApiOperation(value = "老师首页预计收入列表")
    @PostMapping("/teacherIncomeList")
    public HttpResponseResult<PageInfo<TeacherIndexWrapper.TeacherIncome>> teacherIncomeList(@RequestBody TeacherIndexWrapper.CourseExposureSearch query) {
        query.setTeacherId(sysUserService.getUserId());
        PageInfo<TeacherIndexWrapper.TeacherIncome> teacherIncomePageInfo = homeService.teacherIncomeList(PageUtil.getPage(query), query);
        teacherIncomePageInfo.setStatInfo(homeService.teacherIncomeSum(query));
        return succeed(teacherIncomePageInfo);
    }

    @PostMapping("/exportStudentPractice")
    @ApiOperation(value = "查询导出首页练习数据学员列表")
    public HttpResponseResult<String> exportExcel(@RequestBody TeacherIndexWrapper.StudentSearch studentSearch) {
        studentSearch.setTeacherId(sysUserService.getUserId());
        studentSearch.setLimit(10000);
        List<TeacherIndexWrapper.StudentPracticeSummaryDto> teacherHomeStudent = homeService.getTeacherHomeStudent(studentSearch);
        if (CollectionUtils.isEmpty(teacherHomeStudent)) {
            throw new BizException("暂无学员练习记录");
        }
        List<TeacherIndexWrapper.StudentPracticeSummaryExportDto> list =
                JSON.parseArray(JSON.toJSONString(teacherHomeStudent), TeacherIndexWrapper.StudentPracticeSummaryExportDto.class);
        Date date = new Date();
        StringBuffer sb = new StringBuffer("/tmp");
        sb.append("/练习记录")
                .append(studentSearch.getStartTime()).append("~")
                .append(studentSearch.getEndTime()).append("_")
                .append(studentSearch.getTeacherId()).append("_")
                .append(System.currentTimeMillis()).append(".xlsx");
        String filePath = sb.toString();
        File srcFile = new File(filePath);
        EasyExcel.write(filePath,TeacherIndexWrapper.StudentPracticeSummaryExportDto.class).sheet("学员练习统计").doWrite(list);
        //上传到oss
        DateFormat dateFormatOss = new SimpleDateFormat("yyyy/MM/dd");
        String dateStrOss = dateFormatOss.format(date);
        dateStrOss = "klx/studentPractice/" + dateStrOss + "/" + DateUtil.getHour(date);
        String excelFilePath = ossPluginContext.getPluginService(TencentOssPlugin.PLUGIN_NAME).uploadFile(dateStrOss, srcFile);
        FileUtils.deleteQuietly(srcFile);
        return HttpResponseResult.succeed(excelFilePath);
    }

}
