package com.yonge.cooleshow.teacher.task;

import com.yonge.cooleshow.biz.dal.entity.TeacherTotal;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.payment.util.DistributedLock;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @Author: liweifan
 * @Data: 2022/3/28 15:50
 */
@RestController
@RequestMapping("${app-config.url.teacher:}/task")
@ApiIgnore
@Slf4j
public class TaskController extends BaseController {
    @Autowired
    private TeacherTotalService teacherTotalService;
    @Autowired
    private CourseGroupService courseGroupService;
    @Autowired
    private CourseScheduleService scheduleService;
    @Autowired
    private LiveRoomService liveRoomService;

    @Autowired
    private DivBackRecordService divBackRecordService;

    @Autowired
    private RedissonClient redissonClient;
    /***
     * 查询所有老师统计数据
     * @author liweifan
     * @updateTime 2022/3/28 10:12
     * @return: com.yonge.cooleshow.common.entity.HttpResponseResult<java.util.List < com.yonge.cooleshow.biz.dal.vo.TeacherTotalVo>>
     */
    @GetMapping("/queryTeacherTotal")
    public HttpResponseResult<List<TeacherTotal>> queryTeacherTotal() {

        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("queryTeacherTotal", () -> {
            try {

                 succeed(teacherTotalService.queryTeacherTotal());
            }catch (Exception e){
                log.error("queryTeacherTotal error", e);
            }


        }, 10L, TimeUnit.SECONDS);
        return succeed();
    }

    /**
     * <p>定时将符合开售日期的未开售的直播课课程组修改为报名中状态-每隔10分钟执行一次
     * <p>定时将符合结束售卖日期的在售卖中的直播课课程组修改为取消或者成课状态状态-每隔10分钟执行一次
     */
    @GetMapping("/opsCourseGroup")
    public HttpResponseResult<Object> opsCourseGroup() {

        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("opsCourseGroup", () -> {
            try {

                courseGroupService.opsCourseGroup();
            }catch (Exception e){
                log.error("opsCourseGroup error", e);
            }


        }, 10L, TimeUnit.SECONDS);
        return HttpResponseResult.succeed();
    }

    @ApiOperation("定时任务-销毁房间-直播间")
    @GetMapping("/destroyExpiredLiveRoom")
    public void destroyExpiredLiveRoom() {
        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("destroyExpiredLiveRoom", () -> {
            try {

                liveRoomService.destroyExpiredLiveRoom();
            }catch (Exception e){
                log.error("destroyExpiredLiveRoom error", e);
            }


        }, 10L, TimeUnit.SECONDS);
    }

    @ApiOperation("定时任务-创建房间-直播间")
    @GetMapping("/createCourseLiveRoom")
    public void createCourseLiveRoom() {

        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("createCourseLiveRoom", () -> {
            try {

                liveRoomService.createCourseLiveRoom();
            }catch (Exception e){
                log.error("createCourseLiveRoom error", e);
            }


        }, 10L, TimeUnit.SECONDS);
    }

    @GetMapping("/getRedisValueByKey")
    public HttpResponseResult<Object> getRedisValueByKey(String key) {
        return succeed(teacherTotalService.getRedisValueByKey(key));
    }

    @GetMapping("/courseRemind")
    public HttpResponseResult<Object> courseRemind() {
        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("courseRemind", () -> {
            try {

                scheduleService.courseRemind();
            }catch (Exception e){
                log.error("courseRemind error", e);
            }


        }, 10L, TimeUnit.SECONDS);
        return HttpResponseResult.succeed();
    }

    @GetMapping("/courseStatus")
    public HttpResponseResult<Object> courseStatus() {

        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("courseStatus", () -> {
            try {

                scheduleService.scheduleTask();
            }catch (Exception e){
                log.error("courseStatus error", e);
            }
        }, 10L, TimeUnit.SECONDS);
        return HttpResponseResult.succeed();
    }

    @GetMapping("/teacherSalary")
    public HttpResponseResult<Object> teacherSalary() {

        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("teacherSalary", () -> {
            try {

                scheduleService.teacherSalaryTask();
            }catch (Exception e){
                log.error("teacherSalary error", e);
            }
        }, 10L, TimeUnit.SECONDS);
        return HttpResponseResult.succeed();
    }


    @GetMapping("/divBack")
    public HttpResponseResult<Object> divBack() {
        // 并发执行锁
        DistributedLock.of(redissonClient).runIfLockCanGet("divBack", () -> {
            try {

                divBackRecordService.divBack();
            }catch (Exception e){
                log.error("divBack error", e);
            }
        }, 10L, TimeUnit.SECONDS);
        return HttpResponseResult.succeed();
    }
}
