package com.yonge.cooleshow.biz.dal.execl.analysis;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.enums.CellExtraTypeEnum;
import com.alibaba.excel.metadata.Sheet;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.microsvc.toolkit.middleware.excel.analysis.AnalysisContext;
import com.microsvc.toolkit.middleware.excel.analysis.DataAnalysis;
import com.microsvc.toolkit.middleware.excel.util.DownloadManager;
import com.yonge.cooleshow.biz.dal.entity.Student;
import com.yonge.cooleshow.biz.dal.entity.TenantAccountRecord;
import com.yonge.cooleshow.biz.dal.enums.PeriodEnum;
import com.yonge.cooleshow.biz.dal.execl.entity.EntitlementsImport;
import com.yonge.cooleshow.biz.dal.execl.entity.TenantAccountRecordImport;
import com.yonge.cooleshow.biz.dal.execl.listener.EntitlementsImportListener;
import com.yonge.cooleshow.biz.dal.execl.listener.TenantAccountRecordImportListener;
import com.yonge.cooleshow.biz.dal.service.EasyExcelService;
import com.yonge.cooleshow.biz.dal.wrapper.StudentWrapper;
import com.yonge.cooleshow.biz.dal.wrapper.TenantAlbumWrapper;
import com.yonge.cooleshow.common.enums.PostStatusEnum;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;


@Slf4j
@Getter
@Setter
public class EntitlementsImportAnalysis extends DataAnalysis<EntitlementsImportAnalysis> {


    private List<EntitlementsImport> entitlementsImports;

    private Map<String, StudentWrapper.SmallStudent> studentMap;

    private Map<Long, List<TenantAlbumWrapper.SmallTenantAlbum>> tenantAlbumMap;

    // Excel解析服务
    private EasyExcelService easyExcelService;

    private final Map<String, EntitlementsImport> recordMap = Maps.newHashMap();
    public EntitlementsImportAnalysis(AnalysisContext context) {
        super(context);
    }

    public static EntitlementsImportAnalysis build(EasyExcelService easyExcelService) {
        EntitlementsImportAnalysis analysis = new EntitlementsImportAnalysis(AnalysisContext.builder().build());
        analysis.setEasyExcelService(easyExcelService);
        analysis.setEntitlementsImports(Lists.newArrayList());
        return analysis;

    }


    /**
     * 读取文件数据到内存
     *
     * @param path 本地文件地址
     */
    @Override
    public void read(String path) {

        File file = new File(path);
        List<Sheet> sheets = EasyExcel.read(file).build().getSheets();
        if (CollectionUtils.isNotEmpty(sheets)) {

            sheets.forEach(item -> {
                // 读取数据
                EasyExcel.read(file, EntitlementsImport.class, new EntitlementsImportListener(getEntitlementsImports(), item.getSheetName()))
                        .extraRead(CellExtraTypeEnum.MERGE).sheet(item.getSheetName()).doRead();
            });

        }

        // 删除本地缓存文件
        DownloadManager.getInstance().deleteOnExit(path);
    }

    /**
     * 校验数据合法
     *
     * @return Map<Integer, String>
     */
    @Override
    public Map<Integer, String> verify() {

        // 加载校验数据参数
        if (Objects.nonNull(getEntitlementsImports())) {
            loadVerifyParamData();
        }

        Map<Integer, String> verifyDataMap = Maps.newLinkedHashMap();

        // 校验每条入库记录数据合法性
        String ret = "";

        for (EntitlementsImport item : getEntitlementsImports()) {

            StudentWrapper.SmallStudent smallStudent = null;
            if (StringUtils.isBlank(item.getName())) {
                ret += "姓名不能为空；";
            }
            if (StringUtils.isBlank(item.getPhone())) {
                ret += "手机号不能为空；";
            } else {
                smallStudent = studentMap.get(item.getPhone());
                if (Objects.isNull(smallStudent)) {
                    ret += "学生信息不存在或已注销；";
                } else if (!smallStudent.getName().equals(item.getName())) {
                    ret += "学生姓名与手机号不匹配；";
                } else {
                    item.setUserId(smallStudent.getUserId());
                }
            }
            if (StringUtils.isBlank(item.getEntitlementName())) {
                ret += "权益类型不能为空；";
            } else if (item.getEntitlementName().equals("机构专辑")) {
                if (StringUtils.isBlank(item.getTenantAlbumName())) {
                    ret += "机构专辑名称不能为空；";
                } else if (smallStudent != null) {
                    if (smallStudent.getTenantId() == null || smallStudent.getTenantId() ==-1) {
                        ret += "学生未绑定机构,不能添加"+item.getEntitlementName()+"；";
                    } else if (MapUtils.isNotEmpty(tenantAlbumMap)) {
                        List<TenantAlbumWrapper.SmallTenantAlbum> tenantAlbums = tenantAlbumMap.get(smallStudent.getTenantId());
                        if (CollectionUtils.isEmpty(tenantAlbums)) {
                            ret += "机构专辑不存在；";
                        } else {
                            List<TenantAlbumWrapper.SmallTenantAlbum> collect = tenantAlbums.stream()
                                    .filter(o -> o.getName().equals(item.getTenantAlbumName())).collect(Collectors.toList());
                            if (CollectionUtils.isEmpty(collect)) {
                                ret += "机构专辑不存在；";
                            }else if (collect.size() > 1) {
                                ret += "机构专辑名称重复；";
                            } else {
                                item.setTenantAlbumId(collect.get(0).getId());
                            }
                        }
                    }
                }
                item.setEntitlementType("TENANT_ALBUM");
            }else {
                if (smallStudent!=null && smallStudent.getTenantId() != null && smallStudent.getTenantId() >0) {
                    ret += "机构学生只能添加机构专辑；";
                } else {
                    // VIP/SVIP/畅学卡/机构专辑
                    switch (item.getEntitlementName()) {
                        case "VIP":
                            item.setEntitlementType("VIP");
                            break;
                        case "SVIP":
                            item.setEntitlementType("SVIP");
                            break;
                        case "畅学卡":
                            item.setEntitlementType("DISCOUNT");
                            break;
                        default:
                            ret += "权益类型错误；";
                    }
                }
            }
            if (item.getPeriod() == null) {
                ret += "周期错误且不能为空；";
            } else if (item.getPeriod() == PeriodEnum.PERPETUAL) {
                if (!"SVIP".equals(item.getEntitlementType()) && !"TENANT_ALBUM".equals(item.getEntitlementType())) {
                    ret += "仅SVIP、机构专辑支持永久；";
                }
                item.setNumber("100");
            } else if (StringUtils.isBlank(item.getNumber())) {
                ret += "数量不能为空；";
            } else {
                try {
                    Integer.parseInt(item.getNumber());
                } catch (NumberFormatException e) {
                    ret += "数量错误；";
                }
            }
            if (StringUtils.isBlank(item.getReason())) {
                ret += "原因不能为空；";
            }
            if (item.getSendPush()== null) {
                ret += "是否发送推送错误且不能为空；";
            }
            if (StringUtils.isNotEmpty(ret)) {
                verifyDataMap.put(item.getRowIndex(), ret);
                ret = ""; // 重置数据状态
            }
        }

        // 数据校验记录信息
        getContext().total(getEntitlementsImports().size()).failure(verifyDataMap.size())
                .setSuccess(getContext().getTotal() - getContext().getFailure());

        return verifyDataMap;
    }

    /**
     * 数据转换
     */
    @Override
    public void convert() {

    }

    /**
     * 加载校验数据参数
     */
    private void loadVerifyParamData() {

        // 手机号， 查询学生用户信息
        List<String> phoneList = getEntitlementsImports().stream().map(EntitlementsImport::getPhone).distinct().collect(Collectors.toList());

        studentMap = easyExcelService.getStudentMap(phoneList);

        // 查询机构专辑
        List<Long> tenantIds = studentMap.values().stream().map(o -> o.getTenantId())
                .filter(Objects::nonNull)
                .filter(o -> o > 0)
                .distinct().collect(Collectors.toList());
        tenantAlbumMap = easyExcelService.getTenantAlbumMap(tenantIds);


    }


    /**
     * 数据对象
     * @return RoomInfoAnalysis
     */
    @Override
    public EntitlementsImportAnalysis get() {
        return this;
    }

}
