package com.yonge.cooleshow.biz.dal.execl.entity;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.converters.Converter;
import com.alibaba.excel.enums.CellDataTypeEnum;
import com.alibaba.excel.metadata.CellData;
import com.alibaba.excel.metadata.GlobalConfiguration;
import com.alibaba.excel.metadata.property.ExcelContentProperty;
import com.yonge.cooleshow.biz.dal.enums.PeriodEnum;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class EntitlementsImport {


    @ExcelProperty("学生姓名")
    private String name;

    @ExcelProperty("手机号")
    private String phone;

    @ExcelProperty("权益类型")
    private String entitlementName;

    @ExcelProperty("专辑名称")
    private String tenantAlbumName;

    @ExcelProperty("数量")
    private String number;

    @ExcelProperty(value = "周期",converter = PeriodEnumConverter.class)
    private PeriodEnum period;

    @ExcelProperty("原因")
    private String reason;

    @ExcelProperty(value = "是否发送推送",converter = BooleanConverter.class)
    private Boolean sendPush;


    @ExcelIgnore
    private Integer rowIndex;
    @ExcelIgnore
    private String sheetName;

    @ExcelIgnore
    private Long userId;

    @ExcelIgnore
    private Long tenantId;

    @ExcelIgnore
    private Long tenantAlbumId;

    @ExcelIgnore
    private String entitlementType;


    public EntitlementsImport rowIndex(Integer rowIndex) {
        this.rowIndex = rowIndex;
        return this;
    }

    public EntitlementsImport sheetName(String sheetName) {
        this.sheetName = sheetName;
        return this;
    }

    public static class PeriodEnumConverter implements Converter<PeriodEnum> {
        @Override
        public Class<PeriodEnum> supportJavaTypeKey() {
            return PeriodEnum.class;
        }

        @Override
        public CellDataTypeEnum supportExcelTypeKey() {
            return CellDataTypeEnum.STRING;
        }

        @Override
        public PeriodEnum convertToJavaData(CellData cellData, ExcelContentProperty contentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            String stringValue = cellData.getStringValue();
            if (StringUtils.isNotEmpty(stringValue)) {
                switch (stringValue) {
                    case "天":
                        return PeriodEnum.DAY;
                    case "月":
                        return PeriodEnum.MONTH;
                    case "年":
                        return PeriodEnum.YEAR;
                    case "永久":
                        return PeriodEnum.PERPETUAL;
                }
            }
            return null;
        }

        @Override
        public CellData convertToExcelData(PeriodEnum value, ExcelContentProperty contentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            return null;
        }
    }


    public static class BooleanConverter implements Converter<Boolean> {

        @Override
        public Class<Boolean> supportJavaTypeKey() {
            return Boolean.class;
        }

        @Override
        public CellDataTypeEnum supportExcelTypeKey() {
            return CellDataTypeEnum.STRING;
        }

        @Override
        public Boolean convertToJavaData(CellData cellData, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            String stringValue = cellData.getStringValue();
            if (StringUtils.isNotEmpty(stringValue)) {
                return "是".equals(stringValue) || "1".equals(stringValue);
            }
            return null;
        }

        @Override
        public CellData convertToExcelData(Boolean aBoolean, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            return null;
        }
    }
}
