package com.yonge.cooleshow.biz.dal.execl.entity;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.converters.Converter;
import com.alibaba.excel.enums.CellDataTypeEnum;
import com.alibaba.excel.metadata.CellData;
import com.alibaba.excel.metadata.GlobalConfiguration;
import com.alibaba.excel.metadata.property.ExcelContentProperty;
import com.yonge.cooleshow.common.enums.ESettlementFrom;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

@Data
public class StudentImportWrapper {

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class StudentImport{
        @ExcelProperty(value = "学生姓名")
        private String userName;

        @ExcelProperty(value = "手机号")
        private String phone;

        @ExcelProperty(value = "声部")
        private String subjectName;

        @ExcelIgnore
        private Long subjectId;

        @ExcelProperty(value = "出生日期")
        private String birthday;

        @ExcelProperty(value = "性别")
        private String gender;

        @ExcelProperty(value = "所属机构")
        private String tenantName;

        @ExcelIgnore
        private Long tenantId;

        @ExcelProperty(value = "小组")
        private String tenantGroupName;

        @ExcelIgnore
        private Long tenantGroupId;

        @ExcelIgnore
        private Long updateBy;

        @ExcelIgnore
        private Long userId;

        @ExcelIgnore
        private Integer rowIndex;

        @ExcelIgnore
        private String sheetName;


        public StudentImport rowIndex(Integer rowIndex) {
            this.rowIndex = rowIndex;
            return this;
        }

        public StudentImport sheetName(String sheetName) {
            this.sheetName = sheetName;
            return this;
        }

    }

    public static class BooleanConvert implements Converter<Boolean> {

        @Override
        public Class<Boolean> supportJavaTypeKey() {
            return Boolean.class;
        }

        @Override
        public CellDataTypeEnum supportExcelTypeKey() {
            return CellDataTypeEnum.STRING;
        }

        @Override
        public Boolean convertToJavaData(CellData cellData, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            String stringValue = cellData.getStringValue();
            if (StringUtils.isNotEmpty(stringValue)) {
                return "是".equals(stringValue) || "1".equals(stringValue);
            }
            return null;
        }

        @Override
        public CellData convertToExcelData(Boolean aBoolean, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            return null;
        }
    }


    public static class SettlementFromConvert implements Converter<ESettlementFrom> {

        @Override
        public Class<ESettlementFrom> supportJavaTypeKey() {
            return ESettlementFrom.class;
        }

        @Override
        public CellDataTypeEnum supportExcelTypeKey() {
            return CellDataTypeEnum.STRING;
        }

        @Override
        public ESettlementFrom convertToJavaData(CellData cellData, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            String stringValue = cellData.getStringValue();
            if (StringUtils.isNotEmpty(stringValue)) {
                if ("不结算".equals(stringValue)) {
                    return ESettlementFrom.NO;
                } else if ("结算给老师".equals(stringValue)) {
                    return ESettlementFrom.TEACHER;
                } else if ("结算给机构".equals(stringValue)) {
                    return ESettlementFrom.TENANT;
                }
            }
            return null;
        }

        @Override
        public CellData<ESettlementFrom> convertToExcelData(ESettlementFrom o, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            return null;
        }
    }
}
