package com.yonge.cooleshow.biz.dal.execl.entity;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.converters.Converter;
import com.alibaba.excel.enums.CellDataTypeEnum;
import com.alibaba.excel.metadata.CellData;
import com.alibaba.excel.metadata.GlobalConfiguration;
import com.alibaba.excel.metadata.property.ExcelContentProperty;
import com.yonge.cooleshow.common.enums.ESettlementFrom;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

@Data
public class TeacherImportWrapper {

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class TeacherImport{
        @ExcelProperty(value = "老师昵称")
        private String username;

        @ExcelProperty(value = "手机号")
        private String phone;

        @ExcelProperty(value = "真实姓名")
        private String realName;

        @ExcelProperty(value = "身份证号")
        private String idCardNo;

        @ExcelProperty(value = "专业技能")
        private String subjectName;

        @ExcelIgnore
        private String subjectId;

        @ExcelProperty(value = "是否测试用户", converter = BooleanConvert.class)
        private Boolean isTestUser;

        @ExcelProperty("所属机构")
        private String tenantName;

        @ExcelIgnore
        private Long tenantId;

        @ExcelProperty(value = "结算方式", converter = SettlementFromConvert.class)
        private ESettlementFrom settlementFrom;

        @ExcelProperty(value = "是否客服", converter = BooleanConvert.class)
        private Boolean customerService;

        @ExcelIgnore
        private Long updateBy;
        @ExcelIgnore
        private Long userId;

        @ExcelIgnore
        private Integer rowIndex;
        @ExcelIgnore
        private String sheetName;


        public TeacherImport rowIndex(Integer rowIndex) {
            this.rowIndex = rowIndex;
            return this;
        }

        public TeacherImport sheetName(String sheetName) {
            this.sheetName = sheetName;
            return this;
        }

    }

    public static class BooleanConvert implements Converter<Boolean> {

        @Override
        public Class<Boolean> supportJavaTypeKey() {
            return Boolean.class;
        }

        @Override
        public CellDataTypeEnum supportExcelTypeKey() {
            return CellDataTypeEnum.STRING;
        }

        @Override
        public Boolean convertToJavaData(CellData cellData, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            String stringValue = cellData.getStringValue();
            if (StringUtils.isNotEmpty(stringValue)) {
                return "是".equals(stringValue) || "1".equals(stringValue);
            }
            return null;
        }

        @Override
        public CellData convertToExcelData(Boolean aBoolean, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            return null;
        }
    }


    public static class SettlementFromConvert implements Converter<ESettlementFrom> {

        @Override
        public Class<ESettlementFrom> supportJavaTypeKey() {
            return ESettlementFrom.class;
        }

        @Override
        public CellDataTypeEnum supportExcelTypeKey() {
            return CellDataTypeEnum.STRING;
        }

        @Override
        public ESettlementFrom convertToJavaData(CellData cellData, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            String stringValue = cellData.getStringValue();
            if (StringUtils.isNotEmpty(stringValue)) {
                if ("不结算".equals(stringValue)) {
                    return ESettlementFrom.NO;
                } else if ("结算给老师".equals(stringValue)) {
                    return ESettlementFrom.TEACHER;
                } else if ("结算给机构".equals(stringValue)) {
                    return ESettlementFrom.TENANT;
                }
            }
            return null;
        }

        @Override
        public CellData<ESettlementFrom> convertToExcelData(ESettlementFrom o, ExcelContentProperty excelContentProperty, GlobalConfiguration globalConfiguration) throws Exception {
            return null;
        }
    }
}
