package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.beust.jcommander.internal.Lists;
import com.microsvc.toolkit.common.webportal.exception.BizException;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dao.MemberPriceSettingsDao;
import com.yonge.cooleshow.biz.dal.dto.search.VipRecordSearch;
import com.yonge.cooleshow.biz.dal.entity.*;
import com.yonge.cooleshow.biz.dal.enums.*;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.*;
import com.yonge.cooleshow.biz.dal.wrapper.VipCardRecordWrapper;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import com.yonge.toolset.thirdparty.message.MessageSenderPluginContext;
import com.yonge.toolset.utils.date.DateUtil;
import org.jetbrains.annotations.NotNull;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.yonge.cooleshow.biz.dal.dto.search.VipCardRecordSearch;
import com.yonge.cooleshow.biz.dal.dao.VipCardRecordDao;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;


@Service
public class VipCardRecordServiceImpl extends ServiceImpl<VipCardRecordDao, VipCardRecord> implements VipCardRecordService {
    private final static Logger log = LoggerFactory.getLogger(VipCardRecordServiceImpl.class);

    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private MemberPriceSettingsDao memberPriceSettingsDao;
    @Autowired
    private StudentService studentService;


    @Autowired
    private TeacherService teacherService;

    @Autowired
    private MemberPriceSettingsService memberPriceSettingsService;
    @Autowired
    private UserOrderDetailService orderDetailService;

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private DiscountCardRecordService discountCardRecordService;

    @Autowired
    private UserTenantAlbumRecordService userTenantAlbumRecordService;


    @Override
    public VipCardRecordVo detail(Long orderDetilId) {
        return baseMapper.detail(orderDetilId, null);
    }

    @Override
    public VipCardRecordVo detail(Long orderDetilId, Long userId) {
        VipCardRecordVo detail = baseMapper.detail(orderDetilId, userId);
        if (null == detail && null != userId) {
            detail = new VipCardRecordVo();

            UserOrderDetailVo orderDetailVo = orderDetailService.detail(orderDetilId);
            VipCardRecord vipCardRecord = buildVipCardRecordByOrderDetail(orderDetailVo);
            if (null == vipCardRecord) {
                return detail;
            }
            BeanUtils.copyProperties(vipCardRecord, detail);
            MemberPriceSettings memberPrice = memberPriceSettingsDao.getMemberPriceByOrderDetilId(orderDetilId);
            detail.setOriginalPrice(memberPrice.getOriginalPrice());
            detail.setSalePrice(memberPrice.getSalePrice());
            detail.setPeriod(memberPrice.getPeriod());
        }
        return detail;
    }

    @Override
    public VipCardRecord buildVipCardRecordByOrderDetail(UserOrderDetailVo orderDetailVo) {
        if (null == orderDetailVo) {
            return null;
        }
        MemberPriceSettingsVo detail = memberPriceSettingsService.detail(orderDetailVo.getBizId());
        if (null == detail) {
            return null;
        }
        // 按时间区分 个数
        Integer timeNum = orderDetailVo.getGoodNum();
        String periodType  = null;

        if (PeriodEnum.DAY.equals(detail.getPeriod())) {
            periodType = PeriodEnum.DAY.getCode();
        } else if (PeriodEnum.MONTH.equals(detail.getPeriod())) {
            periodType = PeriodEnum.MONTH.getCode();
        } else if (PeriodEnum.QUARTERLY.equals(detail.getPeriod())) {
            timeNum = timeNum*3;
            periodType = PeriodEnum.MONTH.getCode();
        } else if (PeriodEnum.YEAR_HALF.equals(detail.getPeriod())) {
            timeNum = timeNum*6;
            periodType = PeriodEnum.MONTH.getCode();
        } else if (PeriodEnum.YEAR.equals(detail.getPeriod())) {
            periodType = PeriodEnum.YEAR.getCode();
        } else {
            periodType = PeriodEnum.PERPETUAL.getCode();

        }

        return getVipCardRecord(orderDetailVo.getUserId(),orderDetailVo.getOrderClient(),orderDetailVo.getOrderNo(),
                                orderDetailVo.getSubOrderNo(),periodType,detail.getId(),timeNum,SourceTypeEnum.ORDER,orderDetailVo.getUserId(),"会员购买");
    }

    @Override
    public  VipCardRecord buildVipCardRecordByOrderDetail(Long userId,ClientEnum client, ActivityReward activityReward,Long activityId,String activityName) {

        return getVipCardRecord(userId,client,activityId.toString(),null,activityReward.getUnit().toString(),
                                null,activityReward.getNum(),SourceTypeEnum.ACTIVITY,userId,activityName);
    }

    @Override
    public VipCardRecord getVipCardRecord(Long userId, ClientEnum client, String orderNo, String subOrderNo, String periodType,
                                          Long memberPriceSettingsId, Integer timeNum, SourceTypeEnum sourceType, Long createBy, String reason) {

        VipCardRecord vipCardRecord = new VipCardRecord();
        vipCardRecord.setUserId(userId);
        vipCardRecord.setOrderNo(orderNo);
        vipCardRecord.setSubOrderNo(subOrderNo);
        vipCardRecord.setVipCardId(memberPriceSettingsId);
        vipCardRecord.setMsgStatus(0);
        vipCardRecord.setSourceType(sourceType);
        vipCardRecord.setClientType(client);
        vipCardRecord.setTimes(timeNum);
        vipCardRecord.setType(PeriodEnum.valueOf(periodType));
        vipCardRecord.setCreateBy(createBy);
        vipCardRecord.setReason(reason);

        MemberPriceSettings memberPriceSettings = memberPriceSettingsDao.selectById(memberPriceSettingsId);
        if (memberPriceSettings == null) {
            return null;
        }

        VipCardRecordWrapper.UserVip userVip = userVipInfo(userId, client);
        boolean flag= false;

        if (userVip.getVipType() == EVipType.NOT_VIP) {
            vipCardRecord.setStartTime(new Date());
        } else if (userVip.getVipType() == EVipType.VIP && memberPriceSettings.getVipType() ==EVipType.VIP){
            vipCardRecord.setStartTime(userVip.getVipEndDate());
            flag= true;
        } else if  (userVip.getVipType() == EVipType.VIP && memberPriceSettings.getVipType() ==EVipType.SVIP){
            vipCardRecord.setStartTime(new Date());
        }else if  (userVip.getVipType() == EVipType.SVIP && memberPriceSettings.getVipType() ==EVipType.VIP){
            if (userVip.getVipEndDate() !=null) {
                flag= true;
                vipCardRecord.setStartTime(userVip.getVipEndDate());
            } else {
                flag= true;
                vipCardRecord.setStartTime(userVip.getSvipEndDate());
            }
        }else if  (userVip.getVipType() == EVipType.SVIP && memberPriceSettings.getVipType() ==EVipType.SVIP){
            flag= true;
            vipCardRecord.setStartTime(userVip.getSvipEndDate());
        }
        Calendar cal = Calendar.getInstance();
        cal.setTime(vipCardRecord.getStartTime());

        if (PeriodEnum.DAY.getCode().equals(periodType)) {
            cal.add(Calendar.DAY_OF_MONTH, timeNum);
        } else if (PeriodEnum.MONTH.getCode().equals(periodType)) {
            cal.add(Calendar.MONTH, timeNum);
        } else if (PeriodEnum.YEAR.getCode().equals(periodType)) {
            cal.add(Calendar.YEAR, timeNum);
        }

        vipCardRecord.setEndTime(cal.getTime());
        if (flag) {
            DateTime dateTime = DateTime.parse(DateUtil.format(vipCardRecord.getStartTime(), "yyyy-MM-dd"))
                .plusDays(1)
                .withHourOfDay(0)
                .withMinuteOfHour(0)
                .withSecondOfMinute(0)
                .withMillisOfSecond(0);
            vipCardRecord.setStartTime(dateTime.toDate());
        }

        return vipCardRecord;
    }

    @Override
    public IPage<VipCardRecordVo> selectPage(IPage<VipCardRecordVo> page, VipCardRecordSearch query) {
        return page.setRecords(baseMapper.selectPage(page, query));
    }

    @Override
    public void pollExpireMsg() {
        //会员提前三天提示
        List<VipCardRecordVo> temporaryRecords = baseMapper.selectTemporaryRecord();
        if(CollectionUtils.isNotEmpty(temporaryRecords)){
            Map<ClientEnum,List<VipCardRecordVo>> map = temporaryRecords.stream().collect(Collectors.groupingBy(VipCardRecordVo::getClientType));
            //区分老师学生
            for (Map.Entry<ClientEnum, List<VipCardRecordVo>> entry : map.entrySet()) {
                List<VipCardRecordVo> list = entry.getValue();
                //区分会员类型
                Map<EVipType,List<VipCardRecordVo>> map1 = list.stream().collect(Collectors.groupingBy(VipCardRecordVo::getVipType));
                map1.forEach((k,v)->{
                    Map<Long, String> receivers = v.stream().collect(Collectors.toMap(VipCardRecordVo::getUserId, VipCardRecordVo::getPhone));
                    temporary3DaysSend(receivers, entry.getKey(), k);
                });
            }
            //更新状态
            List<Long> ids = temporaryRecords.stream().map(VipCardRecordVo::getId).collect(Collectors.toList());
            baseMapper.updateMsgStatus(ids,1);
        }

        //会员到期提示
        List<VipCardRecordVo> expireRecords = baseMapper.selectExpireRecord();
        if (CollectionUtils.isNotEmpty(expireRecords)){
            Map<ClientEnum,List<VipCardRecordVo>> map = expireRecords.stream().collect(Collectors.groupingBy(VipCardRecordVo::getClientType));
            //区分老师学生
            for (Map.Entry<ClientEnum, List<VipCardRecordVo>> entry : map.entrySet()) {
                List<VipCardRecordVo> list = entry.getValue();
                //区分会员类型
                Map<EVipType,List<VipCardRecordVo>> map1 = list.stream().collect(Collectors.groupingBy(VipCardRecordVo::getVipType));
                map1.forEach((k,v)->{
                    Map<Long, String> receivers = v.stream().collect(Collectors.toMap(VipCardRecordVo::getUserId, VipCardRecordVo::getPhone));
                    expireSend(receivers, entry.getKey(), k);
                });
            }
            //更新状态
            List<Long> ids = expireRecords.stream().map(VipCardRecordVo::getId).collect(Collectors.toList());
            baseMapper.updateMsgStatus(ids,2);
        }
    }

    @Override
    public PageInfo<VipRecordVo> vipRecord(VipRecordSearch recordSearch) {
        return PageUtil.pageInfo(baseMapper.selectVipRecord(PageUtil.getPage(recordSearch),recordSearch));


    }

    /**
     * 获取生效中的会员记录
     * @param userId
     * @param clientEnum
     * @return
     */
    @Override
    public List<VipCardRecord> getEfficientVipRecord(List<Long> userId, ClientEnum clientEnum) {
        if (CollectionUtils.isEmpty(userId)) {
            return new ArrayList<>();
        }
        return this.lambdaQuery()
            .in(VipCardRecord::getUserId, userId)
            .eq(VipCardRecord::getEfficientFlag, true)
            .eq(VipCardRecord::getClientType, clientEnum.name())
            .in(VipCardRecord::getVipType, Lists.newArrayList(EVipType.VIP,EVipType.SVIP))
            .and(wrapper -> wrapper
                .gt(VipCardRecord::getEndTime, new Date())
            )
            .list();
    }

    @Override
    public VipCardRecordWrapper.UserVip UserVipInfo(Long userId, ClientEnum clientEnum) {
        return userVipInfo(userId,clientEnum);
    }
    /**
     * 用户会员信息
     *
     * @param userId
     * @param clientEnum
     * @return
     */
    @Override
    public VipCardRecordWrapper.UserVip userVipInfo(Long userId, ClientEnum clientEnum) {
        // 获取生效中的会员记录
        VipCardRecordWrapper.UserVip userVip = new VipCardRecordWrapper.UserVip();
        List<VipCardRecord> vipCardRecords = this.getEfficientVipRecord(Lists.newArrayList(userId),clientEnum);
        if (CollectionUtils.isEmpty(vipCardRecords)) {
            userVip.setVipType(EVipType.NOT_VIP);
            // 判断有没有过期的会员类型
            Integer vipCount = this.lambdaQuery()
                .eq(VipCardRecord::getUserId, userId)
                .eq(VipCardRecord::getEfficientFlag, true)
                .eq(VipCardRecord::getVipType, EVipType.VIP)
                .eq(VipCardRecord::getClientType, clientEnum.name())
                .count();
            Integer svipCount = this.lambdaQuery()
                .eq(VipCardRecord::getUserId, userId)
                .eq(VipCardRecord::getEfficientFlag, true)
                .eq(VipCardRecord::getVipType, EVipType.SVIP)
                .eq(VipCardRecord::getClientType, clientEnum.name())
                .count();
            if (vipCount>0&&svipCount>0) {
                userVip.setExpireVipType(EVipType.ALL_VIP);
            } else if (vipCount>0) {
                userVip.setExpireVipType(EVipType.VIP);
            } else if (svipCount>0) {
                userVip.setExpireVipType(EVipType.SVIP);
            }
        } else {
            // 存在没有结束时间的SVIP记录 永久SVIP
            List<VipCardRecord> svipList = vipCardRecords.stream().filter(o -> o.getVipType() == EVipType.SVIP).collect(Collectors.toList());
            Date svipEndDate = null;
            if (CollectionUtils.isNotEmpty(svipList)) {
                userVip.setVipType(EVipType.SVIP);

                Optional<VipCardRecord> first = svipList.stream().filter(o ->o.getType()==PeriodEnum.PERPETUAL).findFirst();
                if (first.isPresent()) {
                    userVip.setVipType(EVipType.PERMANENT_SVIP);
                }
//                else {
                Optional<VipCardRecord> max = svipList.stream().filter(o -> !PeriodEnum.PERPETUAL.equals(o.getType())).max(Comparator.comparing(VipCardRecord::getEndTime));
                max.ifPresent(vipCardRecord -> userVip.setSvipEndDate(vipCardRecord.getEndTime()));
//                }
                Optional<VipCardRecord> max1 = svipList.stream().max(Comparator.comparing(VipCardRecord::getEndTime));
                if (max1.isPresent()) {
                    svipEndDate = max1.get().getEndTime();
                }
            } else {
                Integer svipCount = this.lambdaQuery()
                    .eq(VipCardRecord::getUserId, userId)
                    .eq(VipCardRecord::getEfficientFlag, true)
                    .eq(VipCardRecord::getVipType, EVipType.SVIP)
                    .eq(VipCardRecord::getClientType, clientEnum.name())
                    .count();
                if (svipCount>0) {
                    userVip.setExpireVipType(EVipType.SVIP);

                }
            }
            // 过滤出VIP数据
            List<VipCardRecord> vipList = vipCardRecords.stream().filter(o -> o.getVipType() == EVipType.VIP).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(vipList)) {
                if (userVip.getVipType() == null) {
                    userVip.setVipType(EVipType.VIP);
                }
                Optional<VipCardRecord> max = vipList.stream().max(Comparator.comparing(VipCardRecord::getEndTime));
                Optional<VipCardRecord> min = vipList.stream().min(Comparator.comparing(VipCardRecord::getStartTime));
                max.ifPresent(vipCardRecord -> userVip.setVipEndDate(vipCardRecord.getEndTime()));
                Date minDate = null;
                if (min.isPresent()) {
                    minDate = min.get().getStartTime();
                }
                // 设置VIP剩余天数
                if (userVip.getVipEndDate() != null) {
                    int num;
                    if (svipEndDate != null && svipEndDate.after(new Date())) {
                        num = DateUtil.daysBetween(svipEndDate, userVip.getVipEndDate());
                    } else if (minDate != null && minDate.after(new Date())){
                        num = DateUtil.daysBetween(minDate, userVip.getVipEndDate());
                    } else {
                        num = DateUtil.daysBetween(new Date(), userVip.getVipEndDate()) +1;
                    }
                    userVip.setVipEndDays(Math.max(num, 0));
                }
            } else {
                Integer vipCount = this.lambdaQuery()
                    .eq(VipCardRecord::getUserId, userId)
                    .eq(VipCardRecord::getEfficientFlag, true)
                    .eq(VipCardRecord::getVipType, EVipType.VIP)
                    .eq(VipCardRecord::getClientType, clientEnum.name())
                    .count();
                if (vipCount>0) {
                    userVip.setExpireVipType(EVipType.VIP);
                }
            }
        }

        if (userVip.getSvipEndDate() != null) {
            int num = DateUtil.daysBetween(new Date(), userVip.getSvipEndDate()) +1;
            userVip.setSvipEndDays(Math.max(num, 0));
        }

        // 设置会员过期时间
        EVipType expireVipType = userVip.getExpireVipType();
        List<VipCardRecordWrapper.UserVipInfo> userVipInfos = this.baseMapper.queryUserVipInfo(Collections.singletonList(userId), clientEnum.getCode());
        if(!userVipInfos.isEmpty()){
            VipCardRecordWrapper.UserVipInfo userVipInfo = userVipInfos.get(0);
            Date date = new Date();
            if (EVipType.VIP.equals(expireVipType) || EVipType.ALL_VIP.equals(expireVipType)) {
                Date vipEndTime = userVipInfo.getVipEndTime();
                if (vipEndTime != null && vipEndTime.before(date)) {
                    double days = Math.ceil((date.getTime() - vipEndTime.getTime()) * 1.0D / (24 * 60 * 60 * 1000));
                    userVip.setVipExpireDays((int) days);
                }
            }
            if (EVipType.SVIP.equals(expireVipType) || EVipType.ALL_VIP.equals(expireVipType)) {
                Date svipEndTime = userVipInfo.getSvipEndTime();
                if (svipEndTime != null && svipEndTime.before(date)) {
                    double days = Math.ceil((date.getTime() - svipEndTime.getTime()) * 1.0D / (24 * 60 * 60 * 1000));
                    userVip.setSvipExpireDays((int) days);
                }
            }
        }
        return userVip;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(VipCardRecordWrapper.AddVipCardRecord addVipCardRecord) {
        ClientEnum clientType = addVipCardRecord.getClientType();
        if (!ClientEnum.STUDENT.equals(clientType) && !ClientEnum.TEACHER.equals(clientType)) {
            throw new BizException("暂不支持非老师学生会员的增加或扣减");
        }
        if (EVipRecordStatus.DEDUCTION.equals(addVipCardRecord.getStatus())) {
            VipCardRecord vipCardRecord = deductVip(addVipCardRecord);
            vipCardRecord.setDisplayFlag(true);
            this.save(vipCardRecord);
        } else if (EVipRecordStatus.ADD.equals(addVipCardRecord.getStatus())) {
            VipCardRecord addVip = addVip(addVipCardRecord);
            this.save(addVip);
            Integer vipDays = addVipCardRecord.getVipDays();
            // 存在转换，先扣除，再添加
            if (vipDays != null && vipDays > 0) {

                // 转换的 ，找到转换的记录，设置为失效，修改时间，添加一条转换后的记录 订单与添加转换订单一致
                List<VipCardRecord> vipCardRecords = getVipCardRecords(addVipCardRecord.getUserId(), clientType, Lists.newArrayList(EVipType.VIP));
                int vipDaysNum = vipDays;
                if (CollectionUtils.isNotEmpty(vipCardRecords)) {
                    // 根据结束时间正序
                    vipCardRecords.sort(Comparator.comparing(VipCardRecord::getEndTime));
                    List<VipCardRecord> vipCardRecordList = new ArrayList<>();
                    for (VipCardRecord vipCardRecord : vipCardRecords) {
                        int i = DateUtil.daysBetweenUp(vipCardRecord.getStartTime(), vipCardRecord.getEndTime());
                        VipCardRecord record = new VipCardRecord();
                        record.setId(vipCardRecord.getId());
                        if (vipDaysNum == 0) {
                            record.setStartTime(addTime(vipCardRecord.getStartTime(), PeriodEnum.DAY, -vipDaysNum,0));
                            record.setEndTime(addTime(vipCardRecord.getEndTime(), PeriodEnum.DAY, -vipDaysNum,1));
                        } else if (i > vipDaysNum) {
                            record.setEndTime(addTime(vipCardRecord.getEndTime(), PeriodEnum.DAY, -vipDaysNum,1));
                            record.setEfficientFlag(true);
                            record.setDisplayFlag(true);
                            record.setReason("后台转换扣减");
                            record.setUpdateTime(new Date());
                        } else {
                            record.setEfficientFlag(false);
                            record.setDeductionStatus(EDeductionStatus.EXPIRED);
                            record.setUpdateTime(new Date());
                            record.setReason("后台转换扣减");
                            vipDaysNum = vipDaysNum - i;
                        }
                        vipCardRecordList.add(record);
                    }
                    this.updateBatchById(vipCardRecordList);
                }


                VipCardRecordWrapper.AddVipCardRecord addRecord = JSON.parseObject(JSON.toJSONString(addVipCardRecord), VipCardRecordWrapper.AddVipCardRecord.class);
                addRecord.setStatus(EVipRecordStatus.ADD);
                addRecord.setVipType(EVipType.SVIP);
                addRecord.setType(PeriodEnum.DAY);
                addRecord.setTimes(vipDays);
                addRecord.setReason("购买SVIP升级原VIP");
                VipCardRecord convertVip = addVip(addRecord);

                convertVip.setDisplayFlag(true);
                convertVip.setStatus(EVipRecordStatus.UPDATE);
                convertVip.setSourceType(SourceTypeEnum.FREE_UPGRADE);
                // 添加转换记录
                this.save(convertVip);

            }
        }

        studentService.updateVipStatus(addVipCardRecord.getUserId());

        if (Boolean.TRUE.equals(addVipCardRecord.getSendMsg())) {
            sendAddVipMsg(addVipCardRecord.getUserId(),addVipCardRecord.getClientType(),addVipCardRecord.getStatus(),
                    addVipCardRecord.getVipType(),addVipCardRecord.getType(),addVipCardRecord.getTimes(),addVipCardRecord.getReason());
        }
    }

    @Override
    public Map<Long, EVipType> getVipTypeMapByUserIds(List<Long> studentIds, ClientEnum client) {

        List<VipCardRecord> vipCardRecords = this.getEfficientVipRecord(studentIds,client);

        // 根据用户ID分组
        Map<Long, List<VipCardRecord>> vipCardRecordMap = vipCardRecords.stream().collect(Collectors.groupingBy(VipCardRecord::getUserId));
        Map<Long, EVipType> vipTypeMap = new HashMap<>();
        vipCardRecordMap.forEach((k, v) -> {
            List<VipCardRecord> svipList = v.stream().filter(o -> o.getVipType() == EVipType.SVIP).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(svipList)) {
                vipTypeMap.put(k, EVipType.SVIP);
            } else {
                List<VipCardRecord> vipList = v.stream().filter(o -> o.getVipType() == EVipType.VIP).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(vipList)) {
                    vipTypeMap.put(k, EVipType.VIP);
                }
            }
        });
        return vipTypeMap;
    }

    // 会员添加
    @Override
    public VipCardRecord addVip(VipCardRecordWrapper.AddVipCardRecord addVipCardRecord) {
        List<VipCardRecord> vipCardRecordList = getVipCardRecords(addVipCardRecord.getUserId(),addVipCardRecord.getClientType(), Lists.newArrayList(EVipType.VIP,EVipType.SVIP));


        EVipType addVipType = addVipCardRecord.getVipType();
        Date now = new Date();
        Date startDate = now;
        // 没有会员信息
        if (vipCardRecordList.isEmpty()) {
            PeriodEnum period = addVipCardRecord.getType();


            VipCardRecord addRecord = JSON.parseObject(JSON.toJSONString(addVipCardRecord), VipCardRecord.class);
            addRecord.setSourceType(addVipCardRecord.getSourceType() !=null?addVipCardRecord.getSourceType():SourceTypeEnum.BACKEND_GIVE);
            addRecord.setStatus(EVipRecordStatus.ADD);
            addRecord.setDisplayFlag(true);
            addRecord.setEfficientFlag(true);
            addRecord.setStartTime(now);
            addRecord.setVersion(2);
            addRecord.setEndTime(addTime(addRecord.getStartTime(), period, addRecord.getTimes(),1));
            formatVipRecordTime(addRecord);

            addRecord.setEstimatedEndTime(addRecord.getEndTime());
            addRecord.setEstimatedStartTime(addRecord.getStartTime());
            return addRecord;
        }

        if (addVipType.equals(EVipType.VIP)) { // vip 放到最后
            startDate = vipCardRecordList.stream().map(VipCardRecord::getEndTime).max(Date::compareTo).get();
        } else if (addVipType.equals(EVipType.SVIP)) { //放到VIP的前面
            Optional<VipCardRecord> max = vipCardRecordList.stream().filter(o -> o.getVipType().equals(EVipType.SVIP)).max(Comparator.comparing(VipCardRecord::getEndTime));
            if (max.isPresent()) {
                startDate = max.get().getEndTime();
            }
        }

        VipCardRecord newRecord = JSON.parseObject(JSON.toJSONString(addVipCardRecord), VipCardRecord.class);
        newRecord.setSourceType(addVipCardRecord.getSourceType() !=null?addVipCardRecord.getSourceType():SourceTypeEnum.BACKEND_GIVE);
        newRecord.setStatus(EVipRecordStatus.ADD);
        newRecord.setStartTime(startDate);
        newRecord.setEndTime(addTime(newRecord.getStartTime(), addVipCardRecord.getType(), addVipCardRecord.getTimes(),1));

        newRecord.setDisplayFlag(true);
        newRecord.setEfficientFlag(true);

        // 如果是SVIP,平移VIP
        List<VipCardRecord> vipList = vipCardRecordList.stream().filter(o -> o.getVipType() == EVipType.VIP).collect(Collectors.toList());
        if (addVipType.equals(EVipType.SVIP) && CollectionUtils.isNotEmpty(vipList)) {
            // 更新开始时间/结束时间
            List<VipCardRecord> updateList = new ArrayList<>();
            for (VipCardRecord vipCardRecord : vipList) {
                formatVipRecordTime(vipCardRecord);

                // 如果开始时间小于当前时间， 切分过去的时间为一段记录，并且不显示，剩余的时间为一段记录，标记为显示并且生效中
                if (vipCardRecord.getStartTime().before(now)) {
                    VipCardRecord record = JSON.parseObject(JSON.toJSONString(vipCardRecord), VipCardRecord.class);
                    record.setStartTime(vipCardRecord.getStartTime());
                    record.setEndTime(addTime(now,PeriodEnum.DAY,-1,1));
                    record.setEfficientFlag(true);
                    if (record.getStartTime().after(record.getEndTime())) {
                        record.setEfficientFlag(false);
                    }
                    record.setDeductionStatus(EDeductionStatus.EXPIRED);
                    record.setDisplayFlag(false);
                    record.setVersion(2);
                    save(record);
                    if (newRecord.getStartTime().after(now)) {
                        vipCardRecord.setStartTime(addTime(now,PeriodEnum.DAY,0,0));
                    } else {
                        vipCardRecord.setStartTime(addTime(now,PeriodEnum.DAY,1,0));
                    }
                    vipCardRecord.setStatus(EVipRecordStatus.UPDATE);
                }

                VipCardRecord record = new VipCardRecord();
                record.setStartTime(addTime(vipCardRecord.getStartTime(), addVipCardRecord.getType(), addVipCardRecord.getTimes(),0));
                record.setEndTime(addTime(vipCardRecord.getEndTime(), addVipCardRecord.getType(), addVipCardRecord.getTimes(),1));
                record.setStatus(vipCardRecord.getStatus());
                record.setId(vipCardRecord.getId());
                updateList.add(record);
            }
            updateBatchById(updateList);
        }
        newRecord.setDeductionStatus(EDeductionStatus.EFFECTIVE);
        newRecord.setVersion(2);
        formatVipRecordTime(newRecord);
        newRecord.setEstimatedEndTime(newRecord.getEndTime());
        newRecord.setEstimatedStartTime(newRecord.getStartTime());
        return newRecord;
    }

    @NotNull
    private List<VipCardRecord> getVipCardRecords(Long userId,ClientEnum client, List<EVipType> vipTypes) {
        List<VipCardRecord> vipCardRecordList = this.lambdaQuery()
                .eq(VipCardRecord::getClientType, client)
                .eq(VipCardRecord::getUserId, userId)
                .in(CollectionUtils.isNotEmpty(vipTypes),VipCardRecord::getVipType, vipTypes)
                .ge(VipCardRecord::getEndTime, new Date())
                .eq(VipCardRecord::getEfficientFlag, true)
                .list()
                .stream()
                .sorted(Comparator.comparing(VipCardRecord::getStartTime))
                .collect(Collectors.toList());
        return vipCardRecordList;
    }

    @Override
    public List<VipCardRecordWrapper.UserVipInfo> queryUserVipInfo(List<Long> userIdList, String clientType) {
        List<VipCardRecordWrapper.UserVipInfo> userVipInfos = baseMapper.queryUserVipInfo(userIdList, clientType);
        Set<Long> userIds = new HashSet<>();
        List<VipCardRecordWrapper.UserVipInfo> newUserVipInfos = new ArrayList<>();
        for (VipCardRecordWrapper.UserVipInfo userVipInfo : userVipInfos) {
            if (userIds.contains(userVipInfo.getUserId())) {
                continue;
            }
            newUserVipInfos.add(userVipInfo);
            userIds.add(userVipInfo.getUserId());
        }
        return newUserVipInfos;
    }


    /**
     * 扣减会员
     *
     * @param result
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deduction(VipCardRecordWrapper.DeductionVipCardRecord result) {

        // 查找被扣除的记录
        VipCardRecord record = this.getById(result.getRecordId());
        if (record == null) {
            throw new BizException("记录不存在");
        }
        if (record.getDeductionTime() != null) {
            throw new BizException("记录已扣除");
        }
        Date endTime = record.getEndTime();
        if (endTime.before(new Date())) {
            throw new BizException("记录已过期");
        }

        // 获取 剩余时长
        int days =0;
        if (record.getStartTime().before(new Date())) {
            days = DateUtil.daysBetweenUp(new Date(), endTime);
        } else {
            days = DateUtil.daysBetweenUp(record.getStartTime(), endTime);
        }
        VipCardRecord vipCardRecord = new VipCardRecord();
        vipCardRecord.setId(record.getId());
        vipCardRecord.setDeductionTime(new Date());
        vipCardRecord.setDeductionReason(result.getReason());
        vipCardRecord.setEndTime(addTime(record.getEndTime(), PeriodEnum.DAY, -days,1));
        vipCardRecord.setDeductionBy(result.getCreateBy());
        vipCardRecord.setDeductionStatus(EDeductionStatus.DEDUCT);
        if (record.getStartTime().after(vipCardRecord.getEndTime())) {
            vipCardRecord.setEfficientFlag(false);
        }
        this.updateById(vipCardRecord);

        // 对比剩余天数

        // 获取其他有效的记录
        List<VipCardRecord> list = this.lambdaQuery()
                .eq(VipCardRecord::getUserId, record.getUserId())
                .gt(VipCardRecord::getEndTime, endTime)
                .eq(VipCardRecord::getEfficientFlag, 1)
                .list();

        Date startTime = record.getStartTime();
        if (startTime.after(new Date())) {
            startTime = addTime(startTime, PeriodEnum.DAY, -1,1);
        } else {
            startTime = addTime(new Date(), PeriodEnum.DAY, -1,1);
        }
        if (CollectionUtils.isNotEmpty(list)) {
            List<VipCardRecord> updateList = new ArrayList<>();
            for (VipCardRecord vipCardRecord1 : list) {
                VipCardRecord cardRecord = new VipCardRecord();
                cardRecord.setId(vipCardRecord1.getId());
                if (vipCardRecord1.getStatus() == EVipRecordStatus.UPDATE) {
                    cardRecord.setStartTime(addTime(vipCardRecord1.getStartTime(), PeriodEnum.DAY, -days,0));
                    cardRecord.setEndTime(addTime(vipCardRecord1.getEndTime(), PeriodEnum.DAY, -days,1));
                } else {
                    cardRecord.setStartTime(startTime);
                    cardRecord.setEndTime(addTime(cardRecord.getStartTime(), vipCardRecord1.getType(), vipCardRecord1.getTimes(), 1));
                }
                formatVipRecordTime(cardRecord);
                updateList.add(cardRecord);
                startTime = cardRecord.getEndTime();
            }
            this.updateBatchById(updateList);
        }


        studentService.updateVipStatus(result.getUserId());
        if (Boolean.TRUE.equals(result.getSendMsg())) {
            sendAddVipMsg(record.getUserId(),record.getClientType(),EVipRecordStatus.DEDUCTION,
                    record.getVipType(),record.getType(),record.getTimes(),result.getReason());
        }
    }


    /**
     *
     *  获取会员信息
     *
     */
    @Override
    public VipCardRecordWrapper.Info info(VipCardRecordWrapper.InfoQuery infoQuery) {
        VipCardRecordWrapper.Info info = new VipCardRecordWrapper.Info();

        // 所有的会员重新生成
        List<VipCardRecord> vipCardRecordList = getVipCardRecords(infoQuery.getUserId(),infoQuery.getClient(), Lists.newArrayList(EVipType.VIP,EVipType.SVIP));
        if (CollectionUtils.isEmpty(vipCardRecordList)) {
            return info;
        }
        // 过滤出会员
        List<VipCardRecord> vipList = vipCardRecordList.stream().filter(o -> o.getVipType() == EVipType.VIP).collect(Collectors.toList());
        // 计算天数
        int vipDays = 0;
        if (CollectionUtils.isNotEmpty(vipList)) {
            vipDays = vipList.stream().map(o-> {
                if (o.getStartTime().before(new Date())) {
                    o.setStartTime(new Date());
                }
                return o;
            }).mapToInt(o -> DateUtil.daysBetweenUp(o.getStartTime(), o.getEndTime())).sum();
        }
        info.setSurplusVipDay(vipDays);
        // 过滤出SVIP
        List<VipCardRecord> svipList = vipCardRecordList.stream().filter(o -> o.getVipType() == EVipType.SVIP && o.getType() !=PeriodEnum.PERPETUAL).collect(Collectors.toList());
        // 计算天数
        int svipDays = 0;
        if (CollectionUtils.isNotEmpty(svipList)) {
            svipDays = svipList.stream().map(o-> {
                if (o.getStartTime().before(new Date())) {
                    o.setStartTime(new Date());
                }
                return o;
            }).mapToInt(o -> DateUtil.daysBetweenUp(o.getStartTime(), o.getEndTime())).sum();
        }
        info.setSurplusSvipDay(svipDays);
        // 过滤出永久SVIP
        List<VipCardRecord> svipPerpetualList = vipCardRecordList.stream().filter(o -> o.getVipType() == EVipType.SVIP && o.getType() ==PeriodEnum.PERPETUAL).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(svipPerpetualList)) {
            info.setPerpetualFlag(true);
        }
        return info;
    }

    @Override
    public IPage<VipCardRecordWrapper.VipCardRecord> selectAdminPage(IPage<VipCardRecordWrapper.VipCardRecord> page, VipCardRecordWrapper.VipQuery query) {
        IPage<VipCardRecordWrapper.VipCardRecord> vipCardRecordIPage = baseMapper.selectAdminPage(page, query);
        List<VipCardRecordWrapper.VipCardRecord> records = vipCardRecordIPage.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return vipCardRecordIPage;
        }
        // 创建人ID集合
        List<Long> createByList = records.stream()
                .map(VipCardRecordWrapper.VipCardRecord::getOperatorId).filter(Objects::nonNull).distinct().collect(Collectors.toList());
        Map<Long, com.yonge.cooleshow.biz.dal.entity.SysUser> userMap = sysUserService.getMapByIds(createByList);
        records.forEach(vipCardRecord -> {

            if (vipCardRecord.getStartTime().before(new Date())) {
                vipCardRecord.setSurplusDay(DateUtil.daysBetweenUp(new Date(), vipCardRecord.getEndTime()));
            } else {
                vipCardRecord.setSurplusDay(DateUtil.daysBetweenUp(vipCardRecord.getStartTime(), vipCardRecord.getEndTime()));
            }

            com.yonge.cooleshow.biz.dal.entity.SysUser sysUser = userMap.get(vipCardRecord.getOperatorId());
            if (sysUser != null) {
                vipCardRecord.setOperatorName(sysUser.getRealName());
            }
        });
        return vipCardRecordIPage;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<Long> updateUserVipStatus() {
        // 更新会员状态
        List<VipCardRecord> vipCardRecords = this.lambdaQuery()
                .eq(VipCardRecord::getEfficientFlag, true)
                .eq(VipCardRecord::getDeductionStatus, EDeductionStatus.EFFECTIVE)
                .lt(VipCardRecord::getEndTime, new Date())
                .list();
        if (CollectionUtils.isNotEmpty(vipCardRecords)) {
            List<VipCardRecord> updateList = new ArrayList<>();
            for (VipCardRecord vipCardRecord : vipCardRecords) {
                VipCardRecord record = new VipCardRecord();
                record.setId(vipCardRecord.getId());
                record.setDeductionStatus(EDeductionStatus.EXPIRED);
                updateList.add(record);
            }
            this.updateBatchById(updateList);
        }
        // 更新畅学卡状态
        List<DiscountCardRecord> discountCardRecords = discountCardRecordService.lambdaQuery()
                .eq(DiscountCardRecord::getEfficientFlag, true)
                .lt(DiscountCardRecord::getEndTime, new Date())
                .eq(DiscountCardRecord::getDeductionStatus, EDeductionStatus.EFFECTIVE)
                .list();
        if (CollectionUtils.isNotEmpty(discountCardRecords)) {
            List<DiscountCardRecord> updateList = new ArrayList<>();
            for (DiscountCardRecord discountCardRecord : discountCardRecords) {
                DiscountCardRecord record = new DiscountCardRecord();
                record.setId(discountCardRecord.getId());
                record.setDeductionStatus(EDeductionStatus.EXPIRED);
                updateList.add(record);
            }
            discountCardRecordService.updateBatchById(updateList);
        }
        // 更新机构专辑状态
        List<UserTenantAlbumRecord> userTenantAlbumRecords = userTenantAlbumRecordService.lambdaQuery()
                .eq(UserTenantAlbumRecord::getEfficientFlag, true)
                .lt(UserTenantAlbumRecord::getEndTime, new Date())
                .eq(UserTenantAlbumRecord::getDeductionStatus, EDeductionStatus.EFFECTIVE)
                .list();
        if (CollectionUtils.isNotEmpty(userTenantAlbumRecords)) {
            List<UserTenantAlbumRecord> updateList = new ArrayList<>();
            for (UserTenantAlbumRecord userTenantAlbumRecord : userTenantAlbumRecords) {
                UserTenantAlbumRecord record = new UserTenantAlbumRecord();
                record.setId(userTenantAlbumRecord.getId());
                record.setDeductionStatus(EDeductionStatus.EXPIRED);
                updateList.add(record);
            }
            userTenantAlbumRecordService.updateBatchById(updateList);
        }

        // 更新学生表 权益字段
        List<Long> userIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(vipCardRecords)) {
            userIds.addAll(vipCardRecords.stream()
                    .filter(o->o.getClientType() == ClientEnum.STUDENT)
                    .map(VipCardRecord::getUserId).distinct().collect(Collectors.toList()));
        }
        if (CollectionUtils.isNotEmpty(discountCardRecords)) {
            userIds.addAll(discountCardRecords.stream()
                    .map(DiscountCardRecord::getUserId).distinct().collect(Collectors.toList()));
        }
        if (CollectionUtils.isNotEmpty(userTenantAlbumRecords)) {
            userIds.addAll(userTenantAlbumRecords.stream()
                    .map(UserTenantAlbumRecord::getUserId).distinct().collect(Collectors.toList()));
        }


        return userIds;

    }


    // 会员扣减
    private VipCardRecord deductVip(VipCardRecordWrapper.AddVipCardRecord addVipCardRecord) {

        // 所有的会员重新生成
        List<VipCardRecord> vipCardRecordList = getVipCardRecords(addVipCardRecord.getUserId(),addVipCardRecord.getClientType(), Lists.newArrayList(EVipType.VIP,EVipType.SVIP));

        // 当前类型的VIP
        PeriodEnum period = addVipCardRecord.getType();
        // 扣减永久
        if (PeriodEnum.PERPETUAL.equals(period)) {
            addVipCardRecord.setTimes(1);
            return deductedSVipPerpetual(addVipCardRecord, vipCardRecordList);
        }

        // 扣减非永久
        List<VipCardRecord> collect = vipCardRecordList.stream()
                .filter(n -> n.getVipType().equals(addVipCardRecord.getVipType()) && !PeriodEnum.PERPETUAL.equals(n.getType())).collect(Collectors.toList());
        if (collect.isEmpty()) {
            throw new BizException("剩余扣减数量不足");
        }

        Date now = new Date();
        LocalDateTime maxEndTime = collect.stream().map(VipCardRecord::getEndTime).max(Comparator.naturalOrder()).get().toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        // 扣减后的开始时间
        LocalDateTime deductedStartTime;
        switch (period) {
            case DAY:
                deductedStartTime = maxEndTime.minusDays(addVipCardRecord.getTimes());
                break;
            case MONTH:
                deductedStartTime = maxEndTime.minusMonths(addVipCardRecord.getTimes());
                break;
            case QUARTERLY:
                deductedStartTime = maxEndTime.minusMonths(addVipCardRecord.getTimes() * 3L);
                break;
            case YEAR_HALF:
                deductedStartTime = maxEndTime.minusMonths(addVipCardRecord.getTimes() * 6L);
                break;
            case YEAR:
                deductedStartTime = maxEndTime.minusYears(addVipCardRecord.getTimes());
                break;
            default:
                throw new BizException("不支持的扣减类型");
        }
        Date deductedStartDate =  Date.from(deductedStartTime.atZone(ZoneId.systemDefault()).toInstant());;

        Date minStartTime = collect.stream().map(VipCardRecord::getStartTime).min(Comparator.naturalOrder()).get();
        // 如果扣减的数量超过1天，则提示扣减数量不足
        if (deductedStartDate.before(minStartTime)) {
            double day = (minStartTime.getTime() - deductedStartDate.getTime()) * 1.0D / (24 * 60 * 60 * 1000);
            if (day > 1.0D) {
                if (addVipCardRecord.isOrderFlag()) {
                    collect.stream().forEach(o->o.setEfficientFlag(false));

                    this.updateBatchById(collect, 100);
                    return null;
                }

                throw new BizException("剩余扣减数量不足");
            }
//            deductedStartDate = minStartTime;
        }

        // 重新计算会员时间,每一条记录置换成一条新的记录，时间区间重新计算
        List<VipCardRecord> updateRecords = new ArrayList<>();
        Long deductMills = null;
        Long endDeductMills = null;

        EVipType vipType = addVipCardRecord.getVipType();
        boolean giveFlag = false; // 扣减后，剩余时间前移，第一条记录赠送当天的时间
        for (VipCardRecord vipCardRecord : vipCardRecordList) {
            Date startTime = vipCardRecord.getStartTime();
            Date endTime = vipCardRecord.getEndTime();
            if (endTime.before(deductedStartDate) || endTime.equals(deductedStartDate)) {
                continue;
            }
            // 扣减vip,跳过SVIP
            if (EVipType.VIP.equals(vipType) && EVipType.SVIP.equals(vipCardRecord.getVipType())) {
                continue;
            }

            Long addId = null;
            // 扣减到当前时间区间
            if ((deductedStartDate.after(startTime) || deductedStartDate.equals(startTime)) && deductedStartDate.before(endTime)) {
                if (deductedStartDate.after(now)) { // 还有剩余天数,不足一天，补充到当前全天
                    VipCardRecord addRecord = JSON.parseObject(JSON.toJSONString(vipCardRecord), VipCardRecord.class);
                    addRecord.setId(null);
                    addRecord.setDisplayFlag(false);
                    addRecord.setEfficientFlag(true);
                    addRecord.setEndTime(formatEnd(deductedStartDate));
                    addRecord.setStatus(EVipRecordStatus.UPDATE);
                    save(addRecord);
                    addId = addRecord.getId();
                }


                deductMills = endTime.getTime() - deductedStartDate.getTime();
                endDeductMills = endTime.getTime() - formatEnd(deductedStartDate).getTime();
            } else {
                if (vipCardRecord.getVipType().equals(vipType) && !PeriodEnum.PERPETUAL.equals(vipCardRecord.getType())) {
                    deductMills = endTime.getTime() - deductedStartDate.getTime();
                    endDeductMills = endTime.getTime() - formatEnd(deductedStartDate).getTime();
                } else {
                    // 有扣减，整体时间前移
                    if (deductMills != null) {
                        VipCardRecord newRecord = JSON.parseObject(JSON.toJSONString(vipCardRecord), VipCardRecord.class);
                        newRecord.setId(null);
                        newRecord.setDisplayFlag(false);
                        newRecord.setEfficientFlag(true);
                        if (!giveFlag) {
                            newRecord.setStartTime(new Date(newRecord.getStartTime().getTime() - deductMills));
                            giveFlag = true;
                        } else {
                            newRecord.setStartTime(new Date(newRecord.getStartTime().getTime() - endDeductMills));
                        }
                        newRecord.setEndTime(new Date(newRecord.getEndTime().getTime() - endDeductMills));
                        newRecord.setStatus(EVipRecordStatus.UPDATE);
                        save(newRecord);
                        addId = newRecord.getId();
                    }
                }
            }

            VipCardRecord updateRecord = JSON.parseObject(JSON.toJSONString(vipCardRecord), VipCardRecord.class);
            updateRecord.setEfficientFlag(false);
            updateRecord.setRefId(addId);
            updateRecords.add(updateRecord);
        }

        this.updateBatchById(updateRecords, 100);

        VipCardRecord vipCardRecord = JSON.parseObject(JSON.toJSONString(addVipCardRecord), VipCardRecord.class);
        vipCardRecord.setEfficientFlag(false);
        vipCardRecord.setStartTime(deductedStartDate);
        vipCardRecord.setEndTime(Date.from(maxEndTime.atZone(ZoneId.systemDefault()).toInstant()));
        vipCardRecord.setStatus(EVipRecordStatus.DEDUCTION);
        return vipCardRecord;
    }

    // 扣减永久SVIP会员
    private VipCardRecord deductedSVipPerpetual(VipCardRecordWrapper.AddVipCardRecord addVipCardRecord, List<VipCardRecord> vipCardRecordList) {
        List<VipCardRecord> collect = vipCardRecordList.stream()
                .filter(n -> n.getVipType().equals(addVipCardRecord.getVipType()) && n.getType().equals(PeriodEnum.PERPETUAL))
                .collect(Collectors.toList());
        if (collect.isEmpty()) {
            throw new BizException("剩余扣减数量不足");
        }


        // 最后一个非永久SVIP的结束时间
        Date lastUnPereutalDate = new Date();
        Date lastEndUnPereutalDate = formatEnd(new Date());
        Long deductMills = null;
        Long endDeductMills = null;
        boolean giveFlag = false;
        for (VipCardRecord vipCardRecord : vipCardRecordList) {
            if (EVipType.SVIP.equals(vipCardRecord.getVipType())) {
                // 获取后续VIP向前平移的开始时间
                if (!PeriodEnum.PERPETUAL.equals(vipCardRecord.getType())) {
                    if (vipCardRecord.getEndTime().after(lastUnPereutalDate)) {
                        lastUnPereutalDate = vipCardRecord.getEndTime();
                        lastEndUnPereutalDate = vipCardRecord.getEndTime();
                    }
                    continue;
                }
                vipCardRecord.setEfficientFlag(false);
                updateById(vipCardRecord);
            } else {
                VipCardRecord newRecord = JSON.parseObject(JSON.toJSONString(vipCardRecord), VipCardRecord.class);
                newRecord.setId(null);
                newRecord.setDisplayFlag(false);
                newRecord.setEfficientFlag(true);
                if (vipCardRecord.getVipType().equals(addVipCardRecord.getVipType()) && !giveFlag) {
                    deductMills = vipCardRecord.getStartTime().getTime() - lastUnPereutalDate.getTime();
                    endDeductMills = vipCardRecord.getStartTime().getTime() - lastEndUnPereutalDate.getTime();
                    newRecord.setStartTime(new Date(newRecord.getStartTime().getTime() - deductMills));
                    giveFlag = true;
                } else {
                    endDeductMills = vipCardRecord.getStartTime().getTime() - (lastEndUnPereutalDate.getTime() +1000);
                    newRecord.setStartTime(new Date(newRecord.getStartTime().getTime() - endDeductMills));
                }
                newRecord.setEndTime(new Date(newRecord.getEndTime().getTime() - endDeductMills));
                lastEndUnPereutalDate = newRecord.getEndTime();
                save(newRecord);
                Long refId = newRecord.getId();

                VipCardRecord updateRecord = JSON.parseObject(JSON.toJSONString(vipCardRecord), VipCardRecord.class);
                updateRecord.setEfficientFlag(false);
                updateRecord.setRefId(refId);
                updateById(updateRecord);
            }
        }
        VipCardRecord vipCardRecord = JSON.parseObject(JSON.toJSONString(addVipCardRecord), VipCardRecord.class);
        vipCardRecord.setEfficientFlag(false);
        vipCardRecord.setStatus(EVipRecordStatus.DEDUCTION);
        return vipCardRecord;
    }

    private Date plusDate(Date start, PeriodEnum period, long times) {
//        Calendar calendar = Calendar.getInstance();
//        calendar.setTime(start);
//        calendar.set(Calendar.HOUR_OF_DAY, 23);
//        calendar.set(Calendar.MINUTE, 59);
//        calendar.set(Calendar.SECOND, 59);
//        calendar.set(Calendar.MILLISECOND, 0);
        LocalDateTime localDateTimeMaxTime =  LocalDateTime.ofInstant(start.toInstant(),ZoneId.systemDefault());

        LocalDateTime end;
        switch (period) {
            case DAY:
                end = localDateTimeMaxTime.plusDays(times);
                break;
            case MONTH:
                end = localDateTimeMaxTime.plusMonths(times);
                break;
            case QUARTERLY:
                end = localDateTimeMaxTime.plusMonths(times * 3);
                break;
            case YEAR_HALF:
                end = localDateTimeMaxTime.plusMonths(times * 6);
                break;
            case YEAR:
                end = localDateTimeMaxTime.plusYears(times);
                break;
            case PERPETUAL:
                // 永久默认给100年
                end = localDateTimeMaxTime.plusYears(100);
                break;
            default:
                throw new BizException("不支持的扣减类型");
        }
        return Date.from(end.atZone(ZoneId.systemDefault()).toInstant());
    }

    // 发送会员到期3天消息推送
    private void temporary3DaysSend(Map<Long, String> receivers, ClientEnum clientType, EVipType vipType) {
        try {
            MessageTypeEnum messageTypeEnum;
            if(clientType.equals(ClientEnum.TEACHER)){
                messageTypeEnum = EVipType.VIP.equals(vipType) ? MessageTypeEnum.TEACHER_VIP_EXPIRE_THIRTY_DAY : MessageTypeEnum.TEACHER_SVIP_EXPIRE_THIRTY_DAY;
            }else {
                messageTypeEnum = EVipType.VIP.equals(vipType) ? MessageTypeEnum.VIP_EXPIRE_THIRTY_DAY : MessageTypeEnum.SVIP_EXPIRE_THIRTY_DAY;
            }
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,messageTypeEnum
                    , receivers, null, 0, null, clientType.getCode());
        } catch (Exception e) {
            log.error("会员到期3天极光消息推送异常");
        }
    }

    // 发送会员到期消息推送
    private void expireSend(Map<Long, String> receivers,ClientEnum userType, EVipType vipType) {
        try {
            MessageTypeEnum messageTypeEnum;
            if(userType.equals(ClientEnum.TEACHER)){
                messageTypeEnum = EVipType.VIP.equals(vipType) ? MessageTypeEnum.TEACHER_VIP_EXPIRE : MessageTypeEnum.TEACHER_SVIP_EXPIRE;
            }else {
                messageTypeEnum = EVipType.VIP.equals(vipType) ? MessageTypeEnum.VIP_EXPIRE : MessageTypeEnum.SVIP_EXPIRE;
            }
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,messageTypeEnum
                    ,receivers, null, 0, null, userType.getCode());
        } catch (Exception e) {
            log.error("会员到期极光消息推送异常");
        }
    }

    // 添加、扣除会员发送推送
    private void sendAddVipMsg(Long userId,ClientEnum client,EVipRecordStatus status,EVipType vipType,PeriodEnum period,Integer times,String reason) {
        SysUser sysUser = sysUserService.getByUserId(userId);
        if (sysUser == null) {
            return;
        }
        MessageTypeEnum messageTypeEnum;
        if (EVipRecordStatus.ADD.equals(status)) {
            if (EVipType.VIP.equals(vipType)) {
                messageTypeEnum = MessageTypeEnum.PLATFORM_ADD_VIP;
            }else {
                if (PeriodEnum.PERPETUAL.equals(period)) {
                    messageTypeEnum = MessageTypeEnum.PLATFORM_ADD_PER_SVIP;
                } else {
                    messageTypeEnum = MessageTypeEnum.PLATFORM_ADD_SVIP;
                }
            }
        } else if (EVipRecordStatus.DEDUCTION.equals(status)) {
            if (EVipType.VIP.equals(vipType)) {
                messageTypeEnum = MessageTypeEnum.PLATFORM_ADD_DUDECT_VIP;
            }else {
                if (PeriodEnum.PERPETUAL.equals(period)) {
                    messageTypeEnum = MessageTypeEnum.PLATFORM_ADD_DUDECT_PER_SVIP;
                } else {
                    messageTypeEnum = MessageTypeEnum.PLATFORM_ADD_DUDECT_SVIP;
                }
            }
        } else {
            throw new BizException("不支持类型");
        }

        Map<Long, String> receivers = new HashMap<>();
        receivers.put(userId, sysUser.getPhone());

        try {
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, messageTypeEnum,
                    receivers, null, 0, null, client.getCode(), String.valueOf(times), period.getMsg(),
                    reason);
        } catch (Exception e) {
            log.error("会员添加消息发送失败 ： {}", e.getMessage());
        }
    }



    private Date addTime(Date time, PeriodEnum period, Integer times, int type) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(time);

        if (PeriodEnum.DAY.equals(period)) {
            cal.add(Calendar.DAY_OF_MONTH, times);
        } else if (PeriodEnum.MONTH.equals(period)) {
            cal.add(Calendar.MONTH, times);
        } else if (PeriodEnum.YEAR.equals(period)) {
            cal.add(Calendar.YEAR, times);
        } else if (PeriodEnum.QUARTERLY.equals(period)) {
            cal.add(Calendar.MONTH, times * 3);
        } else if (PeriodEnum.YEAR_HALF.equals(period)) {
            cal.add(Calendar.MONTH, times * 6);
        } else if (PeriodEnum.PERPETUAL.equals(period)) {
            cal.add(Calendar.YEAR, 100);
        }
        if (type ==0) {
            cal.set(Calendar.HOUR_OF_DAY, 0);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
        } else {
            cal.set(Calendar.HOUR_OF_DAY, 23);
            cal.set(Calendar.MINUTE, 59);
            cal.set(Calendar.SECOND, 59);
            cal.set(Calendar.MILLISECOND, 0);
        }
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }
    private Date formatEnd(Date date) {
        Calendar c1 = Calendar.getInstance();
        c1.setTime(date);
        c1.set(Calendar.HOUR_OF_DAY, 23);
        c1.set(Calendar.MINUTE, 59);
        c1.set(Calendar.SECOND, 59);
        return c1.getTime();
    }

    private Date formatStart(Date date) {
        Calendar c1 = Calendar.getInstance();
        c1.setTime(date);
        c1.set(Calendar.HOUR_OF_DAY, 0);
        c1.set(Calendar.MINUTE, 0);
        c1.set(Calendar.SECOND, 0);
        c1.set(Calendar.MILLISECOND, 0);
        return c1.getTime();
    }



    public VipCardRecord formatVipRecordTime(VipCardRecord vipCardRecord) {
        // 如果开始时间是23:59:59，开始时间改为第二天的00:00:00
        LocalDateTime startTime = vipCardRecord.getStartTime().toInstant()
                .atZone(ZoneId.systemDefault()).toLocalDateTime();
        if (startTime.toLocalTime().equals(LocalTime.of(23, 59, 59))) {
            startTime = startTime.plusDays(1);
        }

        startTime = startTime.withHour(0).withMinute(0).withSecond(0).withNano(0);
        // 如果结束时间是00:00:00，开始时间改为前一天的23:59:59
        LocalDateTime endTime = vipCardRecord.getEndTime().toInstant()
                .atZone(ZoneId.systemDefault()).toLocalDateTime();
        if (endTime.toLocalTime().equals(LocalTime.of(0, 0, 0))) {
            endTime = endTime.plusDays(-1);
        }
        endTime = endTime.withHour(23).withMinute(59).withSecond(59).withNano(0);
        vipCardRecord.setStartTime(Date.from(startTime.atZone(ZoneId.systemDefault()).toInstant()));
        vipCardRecord.setEndTime(Date.from(endTime.atZone(ZoneId.systemDefault()).toInstant()));
        if (vipCardRecord.getStartTime().after(vipCardRecord.getEndTime())) {
            vipCardRecord.setEndTime(vipCardRecord.getStartTime());
        }
        return vipCardRecord;
    }
}