package com.yonge.cooleshow.biz.dal.wrapper;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.yonge.cooleshow.biz.dal.enums.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang.StringUtils;

import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.Optional;

/**
 * 购买会员卡记录表
 * 2024-12-18 14:24:32
 */
@ApiModel(value = "DiscountCardRecordWrapper对象", description = "购买会员卡记录表查询对象")
public class DiscountCardRecordWrapper {

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" DiscountCardRecordQuery-购买会员卡记录表")
    public static class DiscountCardRecordQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;


        @ApiModelProperty("扣减状态 生效中：EFFECTIVE 过期 EXPIRED 扣减 DEDUCT")
        private EDeductionStatus deductionStatus;

        @ApiModelProperty("关键字匹配")
        private String keyword;

        @ApiModelProperty("用户id")
        @NotNull(message = "用户id不能为空")
        private Long userId;


        @ApiModelProperty("来源类型 ： ACTIVITY ：活动 ，ORDER:订单")
        private SourceTypeEnum sourceType;

        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date endTime;


        @ApiModelProperty("扣减开始时间")
        private Date deductionStartTime;


        @ApiModelProperty("扣减结束时间")
        private Date deductionEndTime;

        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static DiscountCardRecordQuery from(String json) {
            return JSON.parseObject(json, DiscountCardRecordQuery.class);
        }
    }

    @Data
    @ApiModel(" DiscountCardRecord-购买会员卡记录表")
    public static class AddDiscountCardRecord {

        @ApiModelProperty("用户id")
        @NotNull(message = "用户id不能为空")
        private Long userId;

        @ApiModelProperty("购买人员类型 TEACHRE :老师端 STUDNET : 学生端")
        private ClientEnum clientType;

        @ApiModelProperty("操作人ID")
        private Long createBy;

        @ApiModelProperty("时间类型 DAY:天 MONTH:月，YEAR:年")
        private PeriodEnum type;

        @ApiModelProperty("添加时间数量")
        private Integer times;

        @ApiModelProperty("备注")
        private String reason;

        @ApiModelProperty("是否发送推送，1：是，0：否")
        private Boolean sendMsg;

    }


    @Data
    @ApiModel(" DeductionDiscountCardRecord")
    public static class DeductionDiscountCardRecord {

        @ApiModelProperty("用户id")
        @NotNull(message = "用户id不能为空")
        private Long userId;

        @ApiModelProperty("购买人员类型 TEACHRE :老师端 STUDNET : 学生端")
        private ClientEnum clientType;

        @ApiModelProperty("操作人ID")
        private Long createBy;

        @ApiModelProperty("记录ID")
        @NotNull(message = "记录ID不能为空")
        private Long recordId;

        @ApiModelProperty("备注")
        private String reason;

        @ApiModelProperty("是否发送推送，1：是，0：否")
        private Boolean sendMsg;

    }


    @Data
    @ApiModel(" DiscountCardRecordChange")
    public static class DiscountCardRecordChange {

        @ApiModelProperty("用户id")
        private Long userId;

        @ApiModelProperty("会员卡id")
        private Long vipCardId;

        @ApiModelProperty("订单号 ，活动id")
        private String orderNo;

        @ApiModelProperty("订单详情号")
        private String subOrderNo;

        @ApiModelProperty("来源类型 ： ACTIVITY ：活动 ，ORDER:订单")
        private SourceTypeEnum sourceType;

        @ApiModelProperty("购买人员类型 TEACHRE :老师端 STUDNET : 学生端")
        private ClientEnum clientType;

        @ApiModelProperty("操作人ID")
        private Long createBy;


        @ApiModelProperty("时间类型 DAY:天 MONTH:月，YEAR:年")
        private PeriodEnum type;

        @ApiModelProperty("添加时间数量")
        private Integer times;

        @ApiModelProperty("备注")
        private String reason;

        @ApiModelProperty("状态 ADD:新增，DEDUCTION:扣减")
        private EVipRecordStatus status;

    }


    @Data
    public static class DiscountCardRecord {

        @ApiModelProperty("会员记录id")
        private String id;

        @ApiModelProperty("用户id")
        private Long userId;

        @ApiModelProperty("订单号 ，活动id")
        private String orderNo;

        @ApiModelProperty("来源类型 ： 后台扣减 PLATFORM_DEDUCT,活动 ACTIVITY ，订单 ORDER，后台赠送 BACKEND_GIVE")
        private SourceTypeEnum sourceType;

        @ApiModelProperty("购买人员类型 TEACHRE :老师端 STUDNET : 学生端")
        private ClientEnum clientType;

        @ApiModelProperty("会员卡开始时间")
        private Date startTime;

        @ApiModelProperty("会员卡结束时间")
        private Date endTime;


        @ApiModelProperty("创建时间")
        private Date createTime;

        @ApiModelProperty("操作人")
        private Long operatorId;


        @ApiModelProperty("操作人")
        private String operatorName;


        @ApiModelProperty("时间类型 DAY:天 MONTH:月，YEAR:年")
        private PeriodEnum type;

        @ApiModelProperty("添加时间数量")
        private Integer times;

        @ApiModelProperty("备注")
        private String reason;


        @ApiModelProperty("扣减状态 生效中：EFFECTIVE 过期 EXPIRED 扣减 DEDUCT")
        private EDeductionStatus deductionStatus;

        @ApiModelProperty("扣减时间")
        private Date deductionTime;


        @ApiModelProperty("剩余天数")
        private Integer surplusDay = 0;


    }

    @Data
    public static class InfoQuery {

        @ApiModelProperty("用户ID")
        private Long userId;

        @ApiModelProperty("类型")
        private ClientEnum client;
    }

    @Data
    public static class Info {

        @ApiModelProperty("剩余天数")
        private Integer surplusDay = 0;
    }
}
