package com.yonge.cooleshow.biz.dal.wrapper;

import com.alibaba.excel.annotation.ExcelProperty;
import com.yonge.toolset.base.page.QueryInfo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@ApiModel(value = "TeacherIndexWrapper", description = "首页数据统计")
public class TeacherIndexWrapper {
    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class SummarySearch {
        @ApiModelProperty("开始时间")
        private String startTime;

        @ApiModelProperty("结束时间")
        private String endTime;

        @ApiModelProperty("乐器")
        private Long subjectId;

        @ApiModelProperty("老师id")
        private Long teacherId;
    }
    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class StudentSearch {
        @ApiModelProperty("开始时间")
        private String startTime;

        @ApiModelProperty("结束时间")
        private String endTime;

        @ApiModelProperty("乐器")
        private Long subjectId;

        @ApiModelProperty("老师id")
        private Long teacherId;

        @ApiModelProperty("排序字段")
        private String sortField;

        @ApiModelProperty("排序方式 ,ASC升序,DESC降序")
        private String sortType;

        @ApiModelProperty("查询条数")
        private Integer limit = 10;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class TeacherPracticeHome{

        @ApiModelProperty("练习统计")
        private PracticeSummaryDto practiceSummary;

        @ApiModelProperty("练习时长")
        private List<PracticeTimeDto> practiceTimes;

        @ApiModelProperty("练习人数")
        private List<PracticeTimeDto> practiceCounts;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class PracticeSummaryDto{
        @ApiModelProperty("练习人数")
        private BigDecimal practiceCount = BigDecimal.ZERO;

        @ApiModelProperty("平均练习时长")
        private BigDecimal averagePracticeTime = BigDecimal.ZERO;

        @ApiModelProperty("练习总时长")
        private BigDecimal totalPracticeTime = BigDecimal.ZERO;
	}

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class PracticeTimeDto {
        @ApiModelProperty("日期")
        private String date;

        @ApiModelProperty("练习时长")
        private Long practiceTime;

    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class StudentPracticeSummaryDto{

        @ApiModelProperty("学员编号")
        private Long userId;

        @ApiModelProperty("学员姓名")
        private String studentName;

        @ApiModelProperty("学员头像")
        private String avatar;

        @ApiModelProperty("乐器")
        private String subjectName;

        @ApiModelProperty("练习总时长")
        private Long totalPracticeTime;

        @ApiModelProperty("练习总时长(格式化)")
        private String totalPracticeTimeStr;

        @ApiModelProperty("练习天数")
        private Integer practiceDays;

        @ApiModelProperty("平均练习时长")
        private Long averagePracticeTime;

        @ApiModelProperty("平均练习时长(格式化)")
        private String averagePracticeTimeStr;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class StudentPracticeSummaryExportDto{

        @ExcelProperty(value = "学员姓名",index = 0)
        private String studentName;

        @ExcelProperty(value = "乐器",index = 1)
        private String subjectName;

        @ExcelProperty(value = "练习总时长",index = 2)
        private String totalPracticeTimeStr;

        @ExcelProperty(value = "练习天数",index = 3)
        private Integer practiceDays;

        @ExcelProperty(value = "平均练习时长",index = 4)
        private String averagePracticeTimeStr;
    }

    @Data
    @Builder
    @NoArgsConstructor
    public static class CourseExposureTotal{

        @ApiModelProperty("日期")
        private String date;

        @ApiModelProperty("曝光数")
        private Integer exposureNum;

        public CourseExposureTotal(String date, Integer exposureNum) {
            this.date = date;
            this.exposureNum = exposureNum;
        }
    }

    @Data
    public static class TeacherIncome{

        @ApiModelProperty("下单时间")
        private Date orderTime;

        @ApiModelProperty("学员姓名")
        private String userName;

        @ApiModelProperty("学员头像")
        private String userAvatar;

        @ApiModelProperty("学员编号")
        private Long userId;

        @ApiModelProperty("声部")
        private String subjectName;

        @ApiModelProperty("数量")
        private Integer bizQuantity;

        @ApiModelProperty("业务id")
        private Long bizId;

        @ApiModelProperty("业务名称")
        private String bizName;

        @ApiModelProperty("业务图片")
        private String bizCover;

        @ApiModelProperty("金额")
        private BigDecimal amount;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class CourseExposureSearch extends QueryInfo {

        @ApiModelProperty("开始时间")
        private String startTime;

        @ApiModelProperty("结束时间")
        private String endTime;

        @ApiModelProperty("类型MUSIC:乐谱，LIVE:直播课，VIDEO：视频课,VIP_COURSE vip课，GROUP 小组课，PRACTICE 趣纠课")
        private String type;

        @ApiModelProperty("老师id")
        private Long teacherId;

        @ApiModelProperty("学员列表")
        private List<Long> studentIds;

    }

}
