package com.yonge.cooleshow.biz.dal.wrapper;

import com.baomidou.mybatisplus.annotation.TableField;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.yonge.cooleshow.biz.dal.enums.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.Optional;

public class VipCardRecordWrapper {

    @Data
    public static class UserVip{


        @ApiModelProperty(value = "vip类型 VIP:会员 SVIP:SVIP，PERMANENT_SVIP:永久SVIP，NOT_VIP:不是vip")
        private EVipType vipType;


        @ApiModelProperty(value = "vip过期类型 VIP:会员 SVIP:SVIP，ALL_VIP:全vip")
        private EVipType expireVipType;


        @ApiModelProperty(value = "vip结束时间")
        private Date vipEndDate;


        @ApiModelProperty(value = "vip剩余天数")
        private Integer vipEndDays =0;

        @ApiModelProperty(value = "svip结束时间")
        private Date svipEndDate;


        @ApiModelProperty(value = "svip剩余天数")
        private Integer svipEndDays =0;

        @ApiModelProperty("vip失效天数")
        private Integer vipExpireDays;

        @ApiModelProperty("Svip失效天数")
        private Integer svipExpireDays;
    }

    @ApiModel("添加/扣减会员")
    @Data
    public static class AddVipCardRecord {

        @ApiModelProperty("用户ID")
        @NotNull(message = "用户不能为空")
        private Long userId;

        @ApiModelProperty("用户类型")
        @NotNull(message = "用户类型不能为空")
        private ClientEnum clientType;

        @ApiModelProperty("状态 ADD:新增")
        @NotNull
        private EVipRecordStatus status;

        @ApiModelProperty("会员类型，VIP,SVIP")
        @NotNull
        private EVipType vipType;

        @ApiModelProperty("时间类型 DAY:天 MONTH:月 ，QUARTERLY : 季度 YEAR_HALF ： 半年 YEAR:年,永久：PERMANENT")
        @NotNull
        private PeriodEnum type;

        @ApiModelProperty("数量")
        @Min(value = 1, message = "最小数量为1")
        @Max(value = 999, message = "最大数量为999")
        private Integer times;

        @ApiModelProperty("扣减原因")
        @NotNull
        private String reason;

        @ApiModelProperty("是否发送推送,是：true,否：false")
        @NotNull
        private Boolean sendMsg;

        @ApiModelProperty(value = "创建人",hidden = true)
        private Long createBy;

        @ApiModelProperty(value = "转换VIP天数",hidden = true)
        private Integer vipDays;


        @ApiModelProperty("来源类型 ： ACTIVITY ：活动 ，ORDER:订单")
        private SourceTypeEnum sourceType;

        private String orderNo;
        private Long vipCardId;
        private String subOrderNo;

        private boolean orderFlag = false;
    }

    @Data
    public static class UserVipInfo {

        @ApiModelProperty("用户ID")
        private Long userId;

        @ApiModelProperty("当前会员类型")
        private EUserVipType currentVipType;

        @ApiModelProperty("VIP的结束时间")
        private Date currentVipSvipEndTime;

        @ApiModelProperty("vip结束时间")
        private Date vipEndTime;

        @ApiModelProperty("svip结束时间")
        private Date svipEndTime;

        @ApiModelProperty("svip结束时间")
        private Date perSvipEndTime;

        public Date getCurrentVipSvipEndTime() {
            if (EUserVipType.NORMAL.equals(currentVipType)) {
                return Optional.ofNullable(vipEndTime).orElse(svipEndTime);
            } else if (EUserVipType.VIP.equals(currentVipType)) {
                return vipEndTime;
            } else {
                if (perSvipEndTime != null) { // 永久SVIP
                    return null;
                } else {
                    return svipEndTime;
                }
            }
        }
    }

    @Data
    public static class DeductionVipCardRecord {

        @ApiModelProperty("用户id")
        @NotNull(message = "用户id不能为空")
        private Long userId;

        @ApiModelProperty("人员类型 TEACHRE :老师端 STUDNET : 学生端")
        @NotNull(message = "人员类型不能为空")
        private ClientEnum clientType;

        @ApiModelProperty("操作人ID")
        private Long createBy;

        @ApiModelProperty("记录ID")
        @NotNull(message = "记录ID不能为空")
        private Long recordId;

        @ApiModelProperty("备注")
        private String reason;

        @ApiModelProperty("是否发送推送，1：是，0：否")
        private Boolean sendMsg;
    }



    @Data
    public static class InfoQuery {

        @ApiModelProperty("用户ID")
        private Long userId;

        @ApiModelProperty("类型")
        private ClientEnum client;
    }

    @Data
    public static class Info {

        @ApiModelProperty("剩余vip天数")
        private Integer surplusVipDay = 0;

        @ApiModelProperty("剩余svip天数")
        private Integer surplusSvipDay = 0;

        @ApiModelProperty("是否永久")
        private Boolean perpetualFlag = false;

    }

    @Data
    public static class VipQuery  implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;

        @ApiModelProperty("关键字匹配")
        private String keyword;

        @ApiModelProperty("用户Id")
        private Long userId;


        @ApiModelProperty("用户类型 学生：STUDENT 老师 ：TEACHER")
        private ClientEnum clientType;

        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date endTime;

        @ApiModelProperty("会员类型 VIP,SVIP")
        private EVipType vipType;

        @ApiModelProperty("ORDER:激活码激活，TENANT：自行购买，BACKEND_GIVE：后台添加")
        private SourceTypeEnum sourceType;

        @ApiModelProperty("扣减状态 生效中：EFFECTIVE 过期 EXPIRED 扣减 DEDUCT")
        private EDeductionStatus deductionStatus;

        @ApiModelProperty("扣减开始时间")
        private Date deductionStartTime;


        @ApiModelProperty("扣减结束时间")
        private Date deductionEndTime;
    }

    @Data
    public static class VipCardRecord {

        @ApiModelProperty("记录id")
        private String id;

        @ApiModelProperty("用户id")
        private Long userId;


        @ApiModelProperty("会员类型 VIP,SVIP")
        private EVipType vipType;

        @ApiModelProperty("版本")
        private Integer version;

        @ApiModelProperty("订单号")
        private String orderNo;

        @ApiModelProperty("ORDER:订单")
        private SourceTypeEnum sourceType;

        @ApiModelProperty("购买人员类型 TEACHRE :老师端 STUDNET : 学生端")
        private ClientEnum clientType;


        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date endTime;

        @ApiModelProperty("创建时间")
        private Date createTime;

        @ApiModelProperty("操作人")
        private Long operatorId;

        @ApiModelProperty("操作人")
        private String operatorName;


        @ApiModelProperty("时间类型 DAY:天 MONTH:月，YEAR:年")
        private PeriodEnum type;

        @ApiModelProperty("添加时间数量")
        private Integer times;

        @ApiModelProperty("备注")
        private String reason;


        @ApiModelProperty("剩余天数")
        private Integer surplusDay = 0;

        @ApiModelProperty("扣减状态")
        private EDeductionStatus deductionStatus;

        @ApiModelProperty("扣减时间")
        private Date deductionTime;

    }
}
